#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" Description: plot time series from a csv file and summarise the graphics
    Usage of:
	csv2plotj2ts.py file --sep=DELIMITER
    Example:
	# CPI/PPI historical comparison
	psql.sh -d ara < cpippi_hist.sql|grep -v row|python3 csv2plotj2ts.py  --columns=series,value,pbdate  --xaxis=pbdate --pivot_group=series --pivot_value=value  --title="CPI/PPI 走勢" --x_fmt='%m/%Y' --extra_xs='pltStyle="classic";renColumns={"PPIFIS_PCTCHG":"PPI","CPIAUCNS_PCTCHG":"CPI"};ylabel="CPI/PPI (%)";endValueTF=True;yfmt="0.2f";' -
	# treasury rates and spread with 2-YAXIS plot
	printf 'select tsy_2_yr,tsy_10_yr,ROUND((tsy_10_yr-tsy_2_yr)::numeric,2) as "SPREAD_2_10YR", pbdate from tsy_hist where pbdate>=20220401 order by pbdate'|psql.sh -d ara|grep -v row |python3 csv2plotj2ts.py - --xaxis=pbdate --extra_xs='sharXTF=True;columns2="SPREAD_2_10YR";kind2="area";color2="lightgray";yfmt=",.2f"' --title="TREASURY 10Y - 2Y SPREAD in Q2/2022"
	# minute data for 3 major market indices
	yh_hist_batch.py --range=1d --gap=1m   --no_database_save ^GSPC ^DJI ^IXIC --output=csv | python3 csv2plotj2ts.py  --columns=close,epochs,ticker  --xaxis=epochs --pivot_group=ticker --pivot_value=close  --title="美股大盤走勢$(date +%m/%d/%Y)" --return_since_inception --x_fmt='%H:%M' --extra_xs='pltStyle="classic";renColumns={"^GSPC":"標普500","^DJI":"道瓊指數","^IXIC":"納斯達克指數"}' -
	# daily data since begining of the year for 3 major market indices
	yh_hist_batch.py --range=20210101, --gap=1d   --no_database_save ^GSPC ^DJI ^IXIC --output=csv | python3 csv2plotj2ts.py  --columns=close,pbdate,ticker  --xaxis=pbdate --pivot_group=ticker --pivot_value=close  --title="美股大盤$(date +%Y)年走勢" --return_since_inception --x_fmt='%m/%d' --extra_xs='pltStyle="classic";renColumns={"^GSPC":"標普500","^DJI":"道瓊指數","^IXIC":"納斯達克指數"}' -

	# OR (directly pulling data)
	python3 csv2plotj2ts.py TCEHY ^HSI ^GSPC --columns=close,pbdate,ticker  --xaxis=pbdate --pivot_group=ticker --pivot_value=close  --title='騰訊/恆生/標普-2021年初至今報酬率' --interpolate --return_since_inception --x_fmt='%m/%d/%y' --extra_xs='ranged="20210101,20210804";gap="1d";pltStyle="classic";renColumns={"TCEHY":"騰訊","^HSI":"恆生指数","^GSPC":"標普500"}'
	python3 csv2plotj2ts.py ^GSPC ^DJI ^IXIC --columns=close,epochs,ticker  --xaxis=epochs --pivot_group=ticker --pivot_value=close  --title='美股大盤走勢' --return_since_inception --x_fmt='%H:%M' --extra_xs='ranged="1d";gap="1m";pltStyle="classic";renColumns={"^GSPC":"標普500","^DJI":"道瓊指數","^IXIC":"納斯達克指數"}' 
	# FROM data file
	# OR (near realtime data)
	yh_hist_batch.py --range=1d  --no_database_save AAPL --output=csv| csv2plotj2ts.py - --columns=close,epochs  --xaxis=epochs --title=Apple
	#== FOR DAILY DAT
	# directly pull data from web and plot 2y-axis via sharex=True (only df.iloc[:,1] will be applied for secondary_y, the right y-axis)
	python3 csv2plotj2ts.py ^GSPC ^VIX --columns=ticker,close,epochs  --xaxis=epochs --pivot_group=ticker --pivot_value=close  --title='Market vs. Volatility' --trendline --extra_xs='sharexTF=True;ranged="3mo";gap="1d"'
	# OR directly pull data from web and calc cumulated-returns of non 'columns2' list then plot 2y-axis via twinx()
	python3 csv2plotj2ts.py ^GSPC ^VIX --columns=ticker,close,epochs  --xaxis=epochs --pivot_group=ticker --pivot_value=close  --title='標普500累積獲利相對恐慌指數圖' --extra_xs='kind2="line";columns2="^VIX";ranged="3mo";gap="1d";renColumns={"^GSPC":"標普500","^VIX":"恐慌指數"};' --return_since_inception
	# OR (stock data with --src and intraday minute data with 2-axis)
	yh_hist_batch.py --range=1d --gap=1m   --no_database_save ^GSPC ^VIX  --output=csv | python3 csv2plotj2ts.py - --columns=ticker,close,epochs  --xaxis=epochs --pivot_group=ticker --pivot_value=close  --title='Market vs. Volatility' --extra_xs='sharexTF=True;'
	# OR (stock data with --src and intraday minute data)
	yh_hist_batch.py --gap=1d --range=3mo  --no_database_save AAPL --output=csv| csv2plotj2ts.py - --columns=open,close,pbdate  --xaxis=pbdate --title=Apple
	# OR (daily return since inception )
	yh_hist_batch.py --gap=1d --range=3mo  --no_database_save AAPL --output=csv| csv2plotj2ts.py - --columns=open,close,pbdate  --xaxis=pbdate --title=Apple --return_since_inception
	# OR (pivot data)
	printf "select m.label as ticker,p.close as price,p.pbdate from prc_hist p,mapping_series_label m where p.name in ('^GSPC','^TWII','000001.SS','^SOX','^DJI') and p.pbdate>20170101 and p.name=m.series order by m.label,p.pbdate" | psql.sh -d ara | grep -v rows  | python3 csv2plotj2ts.py --pivot_group=ticker --pivot_value=price  --title='Market Overview 2018-05-25' --interpolate --return_since_inception -
	# OR (pivot data and near realtime per minute)
	yh_hist_batch.py --range=1d  --no_database_save AAPL XLK SPY --output=csv| csv2plotj2ts.py - --columns=ticker,close,epochs  --xaxis=epochs --pivot_group=ticker --pivot_value=close  --title='Market Closing Overview' --interpolate --return_since_inception --trendline
	# OR (pivot data with minute data)
	csv2plotj2ts.py CRWD --extra_xs='columns2="volume";gap="1m";renColumns={"close":"CRWD"}' --columns=close,volume --xaxis=epochs --x_fmt='%H:%M' --title="CrowdStrike Intraday"
	# OR (fred data with --src)
	csv2plotj2ts.py DGS2 --src=fred --columns=close,pbdate
	# OR (stock data with --src and candlestick graph)
	csv2plotj2ts.py IBM --src=yh --columns=close,open,high,low,volume,pbdate --title="IBM OHLC" --days=90 --ohlc
	# OR get data from an CSV-formatted text-file
	python3 csv2plotj2ts.py AAPL_XLK_SPY.dat --src=filename --columns=ticker,close,epochs  --xaxis=epochs --pivot_group=ticker --pivot_value=close  --title='Market Closing Overview' --interpolate --return_since_inception --trendline
	# OR (minute data and candlestick graph)
	yh_hist_batch.py --gap=1m --range=1d --no_database_save --output=csv AAPL| csv2plotj2ts.py - --columns=close,open,high,low,volume,epochs,ticker --ohlc --title="Intraday AAPL OHLC"  --xaxis=epochs --trendline
	# OR (input minute data and candlestick Combo graph)
	yh_hist_batch.py --gap=1m --range=1d --no_database_save --output=csv AAPL| csv2plotj2ts.py - --columns=ticker,close,open,high,low,volume,epochs --ohlc_combo --title="Intraday AAPL"   --xaxis=epochs --trendline 
	# OR (src minute data and candlestick Combo graph)
	csv2plotj2ts.py AAPL --src=yh --extra_xs='ranged="1d";gap="1m"' --columns=close,open,high,low,volume,epochs --ohlc_combo --title="Intraday AAPL OHLC"  --xaxis=epochs --trendline
	# OR run specific function like "get_csv2plot"
	yh_hist_batch.py --gap=1m --range=1d --no_database_save --output=csv AAPL| csv2plotj2ts.py - --columns=close,open,high,low,volume,epochs,ticker --ohlc --title="Intraday AAPL OHLC"  --xaxis=epochs --trendline --extra_xs='run="get_csv2plot"' 

    Note: return_since_inception will use $1 as the initial investment if the initial is less than $1
    Last mod., Tue Nov  3 13:40:38 EST 2020
	+ add `endValueTF` option for `extra_xs`
    Last mod., Mon Sep 12 14:34:50 EDT 2022
"""

import sys
from optparse import OptionParser
from scipy.interpolate import interp1d
import pandas as pd
import json
from datetime import datetime
from pandas.plotting import register_matplotlib_converters
from _alan_calc import pqint,getKeyVal,renameDict,subDF
register_matplotlib_converters()
if sys.version_info.major == 2:
	reload(sys)
	sys.setdefaultencoding('utf8')

def subDict(myDict,kyLst,reverseTF=False):
	if reverseTF is True: # invert-match, select non-matching [kyLst] keys 
		return { ky:myDict[ky] for ky in myDict.keys() if ky not in kyLst }
	else:
		return { ky:myDict[ky] for ky in myDict.keys() if ky in kyLst }

def ymd_parser(x,fmt='%Y%m%d'): return datetime.strptime(str(x),fmt)

def epoch_parser(x,s=1000): return datetime.fromtimestamp(int(x/s))

def extrapolate_series(yo):
	yg=yo.dropna()
	#ygi = [int(x) for x in yg.index.values]
	#fn = interp1d(ygi, yg.values, fill_value='extrapolate')
	#yoi = [int(x) for x in yo.index.values]
	#return fn((yoi)
	fn = interp1d(list(map(int,yg.index.values)), yg.values, fill_value='extrapolate')
	return fn(list(map(int,yo.index.values)))


def get_csvdata(args,sep='|',src=None,days=730,start=None,end=None,colLst=None,hdrLst=None,**optx):
	"""
	Get data in datafram with selected [colLst]
	"""
	if isinstance(args,pd.DataFrame):
		df = args
		if colLst is not None and df.size > 0:
			df =  df[ list(set(df.columns) & set(colLst.split(','))) ]
		if hdrLst is not None:
			xLst,yLst = hdrLst.split('=')
			xyD = dict(zip(xLst.split(','),yLst.split(',')))
			df.rename(columns=xyD,inplace=True)
		return df
	if len(args)<1:
		return None
	filename=args[0]
	if filename=='-':
		df=pd.read_csv(sys.stdin,sep=sep)
	elif src.upper()=='FILENAME':
		df=pd.read_csv(filename,sep=sep)
	elif len(args)>1 and src=='yh':
		# TBD, only pull one ticker at a time for now
		#from yh_chart import yh_spark_hist as ysh;
		#ranged=optx.pop('ranged','1d')
		#gap=optx.pop('gap','1m')
		#optx.update(range=ranged,interval=gap,dfTF=True,tsTF=True)
		#df=ysh(tkLst=args,types='chart',**optx)
		from yh_hist_batch import batch_yh_hist
		df = batch_yh_hist(args,days=days,src=src,start=start,end=end,**optx)
		if 'ticker' not in df and 'name' in df:
			df=renameDict(df,{"name":"ticker"})
	elif src is not None:
		from _alan_calc import pull_stock_data
		df = pull_stock_data(filename,days=days,src=src,start=start,end=end,**optx)
	else:
		df = pd.read_csv(filename,sep=sep)
	if df.size < 1:
		sys.stderr.write("**ERROR:{} @{}()".format('Data Not Found!','get_csvdata'))
		return {}
	if colLst is not None:
		df =  df[ list(set(df.columns) & set(colLst.split(','))) ]
	df.dropna(inplace=True)
	if hdrLst is not None:
		xLst,yLst = hdrLst.split('=')
		xyD = dict(zip(xLst.split(','),yLst.split(',')))
		df.rename(columns=xyD,inplace=True)
	return df

def vertex_mnmx(dh,prc_open):
	dmx = dh.loc[dh['actual'] == dh['actual'].max()].iloc[0].to_dict()
	dmn = dh.loc[dh['actual'] == dh['actual'].min()].iloc[0].to_dict()
	dk = []
	if dmx['iptday']>dmn['iptday']:
		dk = dk + [dmn, dmx]
	else:
		dk = dk + [dmx, dmn]
	if dk[0]['iptday'] > dh.loc[0,'iptday']: 
		dk = [dh.iloc[0].to_dict()] + dk
	if dk[-1]['iptday'] < dh['iptday'].iloc[-1]: 
		dk = dk + [dh.iloc[-1].to_dict()]	
	dk[0]['actual'] = prc_open
	dk = pd.DataFrame(dk)
	dk['pchg_open'] = [ x/prc_open-1 for x in dk['actual']]
	dk['sign'] = [ 1 if x>0 else -1  for x in dk['pchg_open']]
	dk['difday'] = dk['iptday'].diff()
	return dk

import matplotlib
if matplotlib.get_backend() != 'TkAgg':
	matplotlib.use('Agg', force=True)
import matplotlib.pyplot as plt
import matplotlib.font_manager as mfm
from matplotlib.ticker import FuncFormatter
font_path = "/usr/share/fonts/truetype/arphic/uming.ttc"
prop = mfm.FontProperties(fname=font_path)

def fontProp(fname=font_path,**optx):
	return mfm.FontProperties(fname=fname,**optx)

def plot_csvdata(df, nbins=6,rsiYN=False,title=None,pivot_value=None,pivot_group=None,pngname=None,x_fmt="%b-%d-%y",interpolateYN=True,backend="tkAgg",npar=15,tsTF=True,xaxis=None,trendTF=False,hiLoTF=False,debugTF=False,ohlcTF=False,ohlcComboTF=False,lang='en',**kwargs):
	if debugTF:
		sys.stderr.write("===plot_csvdata VARS:\t{}\n".format(locals()))
	import matplotlib.dates as mdates
	import matplotlib.image as mimage
	import matplotlib.ticker as mticker
	#pltStyle=getKeyVal(kwargs,'pltStyle','dark_background')
	pltStyle=getKeyVal(kwargs,'pltStyle','classic')
	figsize=getKeyVal(kwargs,'figsize',(11,6))
	ylabel=getKeyVal(kwargs,'ylabel','')
	fontpath=getKeyVal(kwargs,'fontpath',None)
	trendName=getKeyVal(kwargs,'trendName',None)
	sharexTF=getKeyVal(kwargs,'sharexTF',False)
	if fontpath is not None:
		prop.set_file(fontpath)
	if pltStyle in plt.style.available:
		plt.style.use(pltStyle)
	#- Use backend to 'tkAgg' for cronjob
	if pngname is None or len(pngname)<=4:
		plt.switch_backend(backend)

	#- Create datetime index
	idxname='date'
	pbname=xaxis
	if isinstance(df.index,pd.DatetimeIndex):
		pass
	elif pbname in df.columns:
		sdate = str(df[pbname].iloc[0])
		if sdate.isdigit() == True:
			if int(sdate)>123456789:
				idxpt=[epoch_parser(x) for x in df[pbname]]
			else:
				idxpt=[ymd_parser(x,fmt="%Y%m%d") for x in df[pbname]]
		else:
			idxpt=[ymd_parser(x,fmt=x_fmt) for x in df[pbname]]
		df.set_index(pd.DatetimeIndex(idxpt),inplace=True)
		df.index.rename(idxname,inplace=True)
		df = df.drop(pbname,1)
	elif idxname in df.columns:
		df[idxname] = pd.to_datetime(df[idxname])
		df.set_index(idxname,inplace=True)
	else:
		df = df.reset_index(drop=True)


	#- Create a pivot table
	if pivot_group in df.columns and pivot_value in df.columns:
		df=df.pivot_table(index='date',columns=pivot_group,values=pivot_value)

	#- Create linear-interpolation for missing data 
	if interpolateYN is True:
		df=df.apply(extrapolate_series,axis=0)


	#- Create 2nd dataframe 'df2' for 2-Yaxis plot
	dux=getKeyVal(kwargs,'columns2',None)
	if debugTF:
		sys.stderr.write("DF:\n{}\n{}\n".format(df.tail(),df.shape))
	if dux is not None:
		colLst2=dux.split(',')
		df2=subDF(df,colLst2)
		df=subDF(df,colLst2,reverseTF=True)
	else:
		df2={}

	#- Create return since inception
	if rsiYN is True:
		de=[] 
		for j in range(df.shape[1]): 
			inix = df.iloc[0,j] if df.iloc[0,j]>1 else 1
			de.append(df.iloc[:,j]/inix*100.-100)
		#de = [df.iloc[:,j]/df.iloc[0,j]*100.-100 for j in range(df.shape[1])] 
		df = pd.concat(de,axis=1)

	#- Rename columns
	renColumns=getKeyVal(kwargs,'renColumns',{})
	if len(renColumns)>0:
		if debugTF:
			sys.stderr.write("Before rename DF:\n{}\n{}:{}\n".format(df.tail(),df.columns,df.shape))
		df=renameDict(df,mapper=renColumns)
		if debugTF:
			sys.stderr.write("After rename DF:\n{}\n{}:{}\n".format(df.tail(),df.columns,df.shape))
		if debugTF and len(df2):
			sys.stderr.write("Before rename DF2:\n{}\n{}:{}\n".format(df2.tail(),df2.columns,df2.shape))
		if len(df2)>0:
			df2=renameDict(df2,mapper=renColumns)
		if debugTF and len(df2):
			sys.stderr.write("After rename DF2:\n{}\n{}:{}\n".format(df2.tail(),df2.columns,df2.shape))

	#- NO PLOTTING, just return data
	if 'plotTF' in kwargs and kwargs['plotTF'] is False:
		return df,{},{}

	#- Create trend curve
	if trendTF:
		try:
			from _alan_pppscf import vertex_locator
			if trendName is None:
				trendName = df.columns[0]
			if debugTF is True:
				sys.stderr.write("{}\n{}\n".format("Trend data:",df[trendName]))
			dg, dh = vertex_locator(df[trendName],npar=npar,debugTF=debugTF)
			if debugTF is True:
				sys.stderr.write("{}\n{}\n".format("Trendline dg:",dg))
		except Exception as e:
			sys.stderr.write("**ERROR: {} @ {}\n".format(str(e),'trendline'))
			dg, dh = {},{}

	if hiLoTF:
		try:
			from _alan_calc import find_mnmx
			if trendName is None:
				trendName = df.columns[0]
			dg = pd.Series(df[trendName].iloc[sorted(set(find_mnmx(df[trendName])))],name='hiLo')
			if debugTF is True:
				sys.stderr.write("{}\n{}\n".format("hiLoline dg:",dg))
		except Exception as e:
			sys.stderr.write("**ERROR: {} @ {}\n".format(str(e),'hiLoline'))
			dg = {}

	if title is None: 
		title="/".join(df.columns).upper()
		if rsiYN is True:
			title += " Return Since Inception"

	#- plot simple line plot
	if tsTF is False:
		df = df.reset_index(drop=True)

	if debugTF is True:
		sys.stderr.write("{}\n".format(df.head()))
		sys.stderr.write("{}\n".format(df.tail()))
	nobs=len(df.index)
	nsp = int(nobs/nbins) if nobs>nbins*2 else nobs
	#ds=[y for j,y in enumerate(df.index) if j%nsp==0]
	#ax=df.plot(xticks=ds,title=title)
	colorUD = ['red','green'] if lang=='cn' else ['green','red']
	if ohlcComboTF is True:
		from alan_plot import plot_candlestickCombo
		from _alan_calc import run_tech
		chartType = 'minute' if pbname == 'epochs' else 'chart'
		#ma1=5;ma2=30
		ma1,ma2=sma=getKeyVal(kwargs,'sma',[5,30])
		datax = run_tech(df, pcol='close',winLst=sma,debugTF=debugTF,nanTF=True)
		if 'open' not in datax:
			return datax, None, None
		fig, axes = plot_candlestickCombo(datax,title,ma1,ma2,block=False,chartType=chartType,trendTF=trendTF,npar=npar,debugTF=debugTF,colorUD=colorUD,title=title)
		#plt.suptitle(title,fontsize=18,fontproperties=prop)
		if pngname is not None and len(pngname)>4 and '.' in pngname:
			ghLst = plt.gcf().canvas.get_supported_filetypes().keys()
			ghx = pngname.split('.')[-1]
			format = ghx.lower() 
			if ghx.lower() in ghLst:
				format = ghx.lower() 
			else:
				format = 'svg'
			pngname = pngname.replace(ghx,'svg')
			plt.savefig(pngname, format=format) #, bbox_inches='tight',dpi=1000)
		# skip the plot if pngname='noshow'
		elif pngname is None:
			plt.show(axes)
		return datax, fig, axes
	fig, ax=plt.subplots(figsize=figsize,sharex=sharexTF)
	if ohlcTF is True:
		if 'marketVolume' in df:
			df.rename(columns={'marketVolume': 'volume'},inplace=True)
		if 'open' not in df and 'close' in df:
			df['open']=df['high']=df['low']=df['close']
		elif 'open' not in df:
			return df, None, None
		from alan_plot import plot_candlestick
		chartType = 'minute' if pbname == 'epochs' else 'chart'
		ax = plot_candlestick(df,tsidx=df.index,chartType=chartType,title=title,block=False,debugTF=debugTF,ax=ax,trendTF=trendTF,npar=npar,colorUD=colorUD)
		x_fmt = "%H:%M" if chartType == 'minute' else x_fmt
	else:
		colorLst=['blue','red','green','salmon','lightgray','cyan']
		if len(df.columns)>1 and sharexTF:
			col2=df.columns[1]
			df.plot(ax=ax,grid=True,color=colorLst,secondary_y=col2,x_compat=True)
			ylabel2=getKeyVal(kwargs,'ylabel2',col2)
			ax.right_ax.set_ylabel(ylabel2,fontproperties=fontProp(size=12))
		else:
			df.plot(ax=ax,grid=True,color=colorLst,x_compat=True)
			#ax.plot(df.index,df)
			#ax.grid()
		#ax=df.plot(figsize=(11,6))
		ax.set_ylabel(df.columns[0])
		yfmt=getKeyVal(kwargs,'yfmt',',.0f')
		ax.yaxis.set_major_formatter(FuncFormatter(lambda x,pos: '{:{}}'.format(x,yfmt)))
		if any([trendTF,hiLoTF]) and len(dg)>0:
                        dg.plot(ax=ax,x_compat=True)
		if len(df2)>0:
			kind2=getKeyVal(kwargs,'kind2','area')
			color2=getKeyVal(kwargs,'color2','green')
			alpha2=getKeyVal(kwargs,'alpha2',0.4)
			yfmt2=getKeyVal(kwargs,'yfmt2',',.2f')
			ylabel2=getKeyVal(kwargs,'ylabel2','%')
			if debugTF:
				sys.stderr.write("DF2:\n{}\n{}\n".format(df2.tail(),df2.shape))
			axv = ax.twinx()
			df2.plot(ax=axv,kind=kind2,color=color2,alpha=alpha2,legend=False,stacked=False)
			#axv.plot(df2.index,df2,color=color2)
			axv.yaxis.set_major_formatter(FuncFormatter(lambda x,pos: '{:{}}'.format(x,yfmt2)))
			if len(ylabel2)>0:
				axv.set_ylabel(ylabel2)
	if len(ylabel)>0:
		ax.set_ylabel(ylabel,fontproperties=fontProp(size=12))
	elif rsiYN is True: # calc Returns Since Incept
		ax.set_ylabel("Returns Since Inception (%)")
	endValueTF=kwargs.pop('endValueTF',False)
	if endValueTF:
		for sx in df:
			plt.annotate("{:{}}".format(df[sx].iloc[-1],yfmt), xy=(1, df[sx].iloc[-1]), xytext=(8, 0),
				xycoords=('axes fraction', 'data'), textcoords='offset points')
	ax.grid(linestyle='dotted',linewidth=0.5)
	if df.index._typ == "datetimeindex":
		if debugTF is True:
			sys.stderr.write("==DF:\n{}\n".format(df.tail()))
		mddfmt=mdates.DateFormatter(x_fmt)
		ax.xaxis.set_major_formatter(mddfmt)
		xtinterval=(df.index[1]-df.index[0])
		if xtinterval.days < 7 and  xtinterval.days>=1 : # daily data
			ax.set_xlim(df.index[0], df.index[-1])
			#ax.xaxis.set_major_locator(mdates.MonthLocator(interval=int(nsp/30.+0.97)))
			bymd = [1,5,10,15,20,25] if nobs<50 else [1,15] if nobs<120 else [1]
			itv = 1 if nobs<160 else int(nsp/30.+0.97)
			xlocator = mdates.MonthLocator(bymonthday=bymd,interval=itv)
			ax.xaxis.set_major_locator(xlocator)
			# check if min/max of xaxis should be included major ticks
			if debugTF is True:
				sys.stderr.write("{} {}\n".format( ax.get_xticks(),ax.get_xlim()))
			xtcks = list(ax.get_xticks())
			x1,x2 = xtcks[:2]
			xmin,xmax = ax.get_xlim()
			if (x1-xmin)>(x2-x1)*0.6:
				xtcks = [xmin] + xtcks
			if (xmax-xtcks[-1])>(x2-x1)*0.6:
				xtcks = xtcks + [xmax]
			ax.set_xticks(xtcks)
			ax.xaxis.set_minor_locator(mdates.MonthLocator(interval=1))
			if debugTF is True:
				sys.stderr.write("{}\n".format(ax.get_xticks()))
			sys.stderr.write("{}\n".format( "Daily data use MonthLocator"))
		elif xtinterval.seconds and xtinterval.seconds < 30: # second data
			locator = mdates.AutoDateLocator()
			locator.intervald[5] = [0,5,10,15,20,25,30,35,40,45,55]
			mddfmt = mdates.AutoDateFormatter(locator)
			mddfmt.scaled[1/(24.*60.)] = '%M:%S' 
			ax.xaxis.set_major_locator(locator)
			ax.xaxis.set_major_formatter(mddfmt)
			sys.stderr.write("{} {}\n".format( "Second data use AutoDateLocator",xtinterval.seconds))
		elif xtinterval.seconds and xtinterval.seconds < 100 : # minute data
			bym = [0,15,30,45] if nobs<=120 else [0,30] if nobs<=360 else [0]
			xlocator = mdates.MinuteLocator(byminute=bym, interval = 1)
			ax.xaxis.set_major_locator(xlocator)
			sys.stderr.write("{} {}\n".format( "Minute data use MinuteLocator",xtinterval.days))
		else: # periodic data
			sys.stderr.write("{} {}\n".format( "Periodic data use DayLocator",nsp))
			#ax.xaxis.set_major_locator(mdates.DayLocator(interval=nsp))
			xlocator = mdates.AutoDateLocator()
			ax.xaxis.set_major_locator(xlocator)
			ax.xaxis.set_minor_locator(mdates.MonthLocator(interval=1))
	ax.xaxis.label.set_visible(False)
	plt.title(title,fontproperties=fontProp(size=30))
	plt.xticks(rotation='20',fontsize=10)
	if len(df.columns)>0 and sharexTF is True:
		h1, l1 = ax.get_legend_handles_labels()
		h2, l2 = ax.right_ax.get_legend_handles_labels()
		if len(ylabel)<1 and len(l1)>0:
			ylabel=l1[0]
		ax.legend(h1+h2, [ylabel]+[ylabel2], loc="upper left",prop=prop)
	elif len(df.columns)>0 and sharexTF is False and ohlcTF is False:
		ax.legend(df.columns,loc="upper left",prop=prop)
	#ax.legend().set_visible(False)
	#logo = mimage.imread("aicaas_icon.png")
	#plt.figimage(logo, xo=20,yo=420)
	plt.subplots_adjust(left=0.1,bottom=0.30)
	if pngname is not None and len(pngname)>4:
		plt.savefig(pngname)#, bbox_inches='tight',dpi=1000)
		sys.stderr.write("Save chart {} to {}\n".format(title,pngname))
	else:
		print("===>ax:",ax)
		plt.show()
		#plt.show(ax)
	return df, fig, [ax]

def run_comment_hourly(df, opts=None):
	from _alan_str import generate_comment_hourly
	if 'ticker' not in opts:
		if 'ticker' in df.columns:
			opts['ticker'] = df['ticker'].iloc[0]
		else:
			return {}
	ohlcComboTF = opts['ohlcComboTF'] if 'ohlcComboTF' in opts else False
	jobj = {}
	if opts['j2ts'] is not None and len(opts['j2ts'])>0:
		ts=opts['j2ts']
	elif 'j2name' in opts and len(opts['j2name'])>0:
		fj2name = "{}_{}.j2".format(opts['j2name'],opts['lang'])
		ts='{} include "{}" {}'.format('{%',fj2name,'%}')
	else:
		ts=''
	if len(ts)<1:
		return {}
	try:
		jobj = generate_comment_hourly(ts,df,opts)
		ret = jobj['comment'] if 'comment' in jobj else ''
		if 'plotTF' in opts and opts['plotTF'] is False:
			return jobj
		if len(ret)>0:
			pqint( ret, file=sys.stdout)
			if 'plotTextTF' in opts and opts['plotTextTF']:
				bottom = 0.35 if ohlcComboTF is True else 0.25
				plt.subplots_adjust(left=0.1,bottom=bottom)
				txbtm = 0.05 if ohlcComboTF is True else 0.1
				plt.text(0.05,txbtm,ret,fontsize=11,color='yellow',fontproperties=prop, transform=plt.gcf().transFigure)
			pngname = opts['pngname']
			if pngname is not None and len(pngname)>4:
				#pngx = pngname.split('.')[0].split('_')[0] + opts['rpt_time'].strftime("_%Y%m%d_%H%M") +'.svg'
				#if opts['pngname'] == pngx:
				#	pqint( "svg filename {} is duplicated!".format(pngx), file=sys.stderr)
				#	return jobj
				#opts['pngname'] = pngx
				outdir = opts['outdir'] if 'outdir' in opts else '.'
				pngpath = "{}/{}".format(outdir,pngname)
				sys.stderr.write("rpt_time:{},svg_path:{}\n".format(opts['rpt_time'],pngpath))
				plt.savefig(pngpath, format='svg')
			else:
				plt.show()
	except Exception as e:
		sys.stderr.write("**ERROR: @ generate_comment_hourly\n{}\n".format(str(e)))
	return jobj

def get_csv2plot(args,opts=None,**optx):
	"""
	plot time series data from csv file 
	"""
	#- Set input parameters
	if opts is None or len(opts)<1:
		opts, _ = opt_csv2plot([])
	if optx is not None:
		opts.update(optx)
	debugTF=opts['debugTF']
	if debugTF:
		sys.stderr.write("===get_csv2plot VARS:\t{}\n".format(locals()))
	#for ky,va in opts.items():
	#	exec("{}=va".format(ky))

	#- Get data in datafram with selected [colLst]
	if isinstance(args,pd.DataFrame) is True: 
		df = args
	else:
		gkys=['sep','src','days','start','end','colLst','hdrLst','debugTF','ranged','gap','tsTF','pchgTF','searchDB','range','interval']
		df = get_csvdata(args, **subDict(opts,gkys))

	if df is None or len(df)<1 or df.size<1:
		return {},{},{}
	if debugTF is True:
		sys.stderr.write("OPTS: {}\n".format(opts))
		sys.stderr.write("DATA:\n{}\n".format(df.head()))

	pkys=['interpolateYN','title','debugTF','tsTF','rsiYN','pivot_value','pivot_group','ohlcTF','ohlcComboTF','xaxis','trendTF','pngname','x_fmt','nbins','npar','backend','lang']
	#df,fig,axes = plot_csvdata(df,**subDict(opts,pkys))
	df,fig,axes = plot_csvdata(df,**opts)
	return df,fig,axes

def run_csv2plot(args,opts=None,**optx):
	pngname=opts['pngname']
	opts['pngname']='noshow'
	df,fig,axes = get_csv2plot(args,opts=opts,**optx)
	opts['pngname']=pngname
	opts['intradayTF'] = True if (df.index[1]-df.index[0]).days<1 else False
	if 'rpt_time' not in opts:
		rpt_time = df.index[-1]
		opts.update(rpt_time=rpt_time)
	try:
		jobj = run_comment_hourly(df, opts)
	except Exception as e:
		sys.stderr.write("**ERROR:{} @{}\n".format(str(e),'run_csv2plot'))
		jobj = {}
	return df, opts, jobj

def opt_csv2plot(argv=[],retParser=False):
	""" command-line options initial setup
	    Arguments:
		argv:	list arguments, usually passed from sys.argv
		retParser:	OptionParser class return flag, default to False
	    Return: (options, args) tuple if retParser is False else OptionParser class 
	"""
	parser = OptionParser(usage="usage: %prog [option] FILENAME", version="%prog 1.0",
		description="Time-series Plotting Utility via matplotlib")
	parser.add_option("-s","--sep",action="store",dest="sep",default="|",
		help="field separator (default: |)")
	parser.add_option("","--xaxis",action="store",dest="xaxis",default="pbdate",
		help="x-axis [pbdate|epochs] column name (default: pbdate in yyyymmdd)")
	parser.add_option("","--columns",action="store",dest="colLst",
		help="selected columns (default: ALL)")
	parser.add_option("","--ren_header",action="store",dest="hdrLst",
		help="rename header columns")
	parser.add_option("-t","--title",action="store",dest="title",
		help="title (default: combo-colunms)")
	parser.add_option("-n","--nbins",action="store",dest="nbins",default="6",type=int,
		help="number of bins in x-axis (default: 6)")
	parser.add_option("","--return_since_inception",action="store_true",dest="rsiYN",default=False,
		help="use Return since Inception plot. Note: $1 will be used as the initial investment if the initial is less than $1")
	parser.add_option("","--interpolate",action="store_true",dest="interpolateYN",default=False,
		help="use linear-interplation for missing data")
	parser.add_option("","--pivot_group",action="store",dest="pivot_group",
		help="pivot table group by column, must pair with PIVOT_VALUE")
	parser.add_option("","--pivot_value",action="store",dest="pivot_value",
		help="pivot table display value column, must pair with PIVOT_GROUP")
	parser.add_option("","--x_fmt",action="store",dest="x_fmt",default='%m-%d-%y',
		help="graph x-axis format (default: %m-%d-%y)")
	parser.add_option("","--png",action="store",dest="pngname",
		help="graph name (default: None)")
	parser.add_option("","--backend",action="store",dest="backend",default='tkAgg',
		help="matplotlib new backend(default: tkAgg)")
	parser.add_option("","--no_time_series",action="store_false",dest="tsTF",default=True,
		help="Simple line plot no time-series")
	parser.add_option("-l","--lang",action="store",dest="lang",default="cn",
		help="language mode [cn|en] (default: cn), ohlc/ohlc_combo ONLY")
	parser.add_option("","--ohlc",action="store_true",dest="ohlcTF",default=False,
		help="plot stock OHLC Candlestick")
	parser.add_option("","--ohlc_combo",action="store_true",dest="ohlcComboTF",default=False,
		help="plot stock OHLC Candlestick + MA/RSI/MACD Combo")
	parser.add_option("","--src",action="store",dest="src",default="yh",
		help="data source (FILENAME is used if 'FILENAME' is provided. default: yh)")
	parser.add_option("","--start",action="store",dest="start",
		help="start YYYY-MM-DD, must pair with SRC (default: 2-years-ago)")
	parser.add_option("","--end",action="store",dest="end",
		help="end YYYY-MM-DD, must pair with SRC (default: today)")
	parser.add_option("","--days",action="store",dest="days",default=730,type=int,
		help="number of days from END date, must pair with SRC (default: 730)")
	parser.add_option("","--trendline",action="store_true",dest="trendTF",default=False,
		help="Draw trend-line, apply to the 1st array ONLY")
	parser.add_option("","--hiloline",action="store_true",dest="hiLoTF",default=False,
		help="Draw hilo-line, apply to the 1st array ONLY")
	parser.add_option("","--npar",action="store",dest="npar",default=15,type="int",
		help="trendline fitting polynomial degree (default: 15)")
	parser.add_option("","--j2ts",action="store",dest="j2ts",
		help="template script. Note, JINJA2 ONLY")
	parser.add_option("","--extra_js",action="store",dest="extraJS",
		help="extra JSON in DICT format.")
	parser.add_option("","--extra_xs",action="store",dest="extraXS",
		help="extra excutable string in k1=v1;k2=v2; format")
	parser.add_option("","--debug",action="store_true",dest="debugTF",default=False,
		help="debugging (default: False)")
	(options, args) = parser.parse_args(argv[1:])
	if retParser is True:
		return parser
	try:
		opts = vars(options)
		from _alan_str import extra_opts
		extra_opts(opts,xkey='extraJS',method='JS',updTF=True)
		extra_opts(opts,xkey='extraXS',method='XS',updTF=True)
	except Exception as e:
		sys.stderr.write("**ERROR:{} @{}\n".format(str(e),'opt_csv2plot'))
	return (opts, args)

def tst_csv2plotj2ts(args,opts=None,**optx):
	try:
		ret = run_csv2plot(args,opts,**optx)
		return ret
	except Exception as e:
		sys.stderr.write("**ERROR:{} @{}\n".format(str(e),'run_csv2plot'))
		return None, ''

if __name__ == '__main__':
	opts, args = opt_csv2plot(sys.argv)
	if 'run' in opts and opts['run'] in globals():
		ret = globals()[opts['run']](args,opts=opts)
	else:
		ret = get_csv2plot(args,opts)
		#ret = tst_csv2plotj2ts(args,opts)



from setuptools import setup,find_packages
VERSION = '0.1.4' 
DESCRIPTION = 'Technical ALgorithmic ANalytics',
LONG_DESCRIPTION = 'Utility funcitions for financial analysis'

# Setting up
setup(
    name='talan',
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
	long_description_content_type='text/markdown',
    url='https://github.com/beyondbond/talan',
    author='Ted Hong',
    author_email='ted@beyondbond.com',
    license='BSD 2-clause',
    packages=['talan'],
    install_requires=['pandas','numpy','seaborn','scipy','matplotlib'],

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Financial and Insurance Industry',
        'License :: OSI Approved :: BSD License',  
        'Operating System :: POSIX :: Linux',        
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.8',
    ],
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Usage of:
python3 _alan_mp4.py daily_briefing --extra_xs="txtOnly=True"
python3 _alan_mp4.py intraday_briefing --extra_xs="txtOnly=True"
OR
python3 _alan_mp4.py daily_single_stock MDB --extra_xs="txtOnly=True;deltaTolerance=43200"
OR
python3 _alan_mp4.py daily_single_stock MDB 1

Description: mp4 factory
Version: 0.9
Last mod., Thu Oct 29 13:19:52 EDT 2020
"""
import sys,os
import re
from optparse import OptionParser
from glob import glob
import numpy as np
import pandas as pd
from _alan_calc import getKeyVal,subDict,saferun
from _alan_str import jj_fmt,upsert_mdb,find_mdb
from srt_convert import generateSubtitle
from subprocess import Popen,PIPE
import datetime
from pandas import Timestamp
if sys.version_info.major == 2:
	reload(sys)
	sys.setdefaultencoding('utf8')

def str2audio(s='',fname='',lang='cn',tempo=1.25,debugTF=False):
	'''
	create a audio file named 'fname' based on string 's'
	'''
	if any([len(s)<1,len(fname)<1]) is True:
		return '','**ERROR: No string for audio'
	glang='zh-CN' if lang=='cn' else lang
	xcmd="/usr/local/bin/gtts-cli -l {} -f - | sox -G -t mp3 - {} tempo {}".format(glang,fname,tempo)
	if debugTF:
		sys.stderr.write("SHELL_CMD:\n{} | {}\n".format(s,xcmd))
	p = Popen(xcmd,shell=True,bufsize=1024,stdout=PIPE,stdin=PIPE,stderr=PIPE)
	o, e = p.communicate(s.encode('utf-8'))
	if debugTF:
		sys.stderr.write("===SHELL_CMD:\n{}\n---OUT:\n{}\n---ERR:\n{}\n".format(xcmd,o,e))
	return o,e

def run_concat(listName='',videoLst=[],videoPath='',outdir='',lang='cn',debugTF=False):
	'''
	concatenate 'videoLst' mp4 files and save to 'videoPath'
	'''

	if len(outdir)<1:
		outdir=os.path.dirname(videoLst[0])
	vlst=[os.path.basename(x) for x in videoLst]
	listStr="file {} \n".format("\nfile ".join(vlst))
	mp4Name=os.path.basename(videoPath)
	if len(listName)<1:
		listName=re.sub('(_ALL)?(.mp4)?','',mp4Name)+'.list'
		listPath="{}/{}".format(outdir,listName)
	fp=open(listPath,"w")
	fp.write(listStr)
	fp.close()
	tempName=re.sub('(_ALL)?(.mp4)?','',mp4Name)+'_temp.mp4'
	sampleRate=24000 if lang =='en' else 22050 
	ffm ='''cd {outdir};'''
	ffm+='''ffmpeg -nostdin -f concat -i "{listName}" -c copy  -y "{tempName}";'''
	ffm+='''ffmpeg -nostdin -i {tempName} -c:v libx264 -pix_fmt yuv420p -flags +global_header -strict -2 -acodec aac -ar {sampleRate} -ab 32k "{mp4Name}" -y'''
	# need listName, tempName, mp4Name 
	xcmd=ffm.format(**locals())
	#return ffm
	try:
		p = Popen(xcmd,shell=True,bufsize=1024,stdout=PIPE,stdin=PIPE,stderr=PIPE)
		o, e = p.communicate()
		#This makes the wait possible
		p_status = p.wait()
	except Exception as e:
		sys.stderr.write("**ERROR: {} @ {}\n".format(str(e),'run_ffmpeg') )
		o=''
	if debugTF:
		sys.stderr.write("===SHELL_CMD:\n{}\n---OUT:\n{}\n---ERR:\n{}\n".format(xcmd,o,e))
	return o,e

def run_ffmpeg(chartPath='',audioPath='',videoPath='',srtPath='',lang='cn',debugTF=False):
	'''
	Create  mp4 file 'videoPath'
	  based on chart,audio and subtitle files w.r.t.
	  'chartPath', 'audioPath', 'srtPath'
	'''
	if lang=='en' :
		fs=22
		fb="blue"
		fc="white"
		spd=3.5
		sampleRate=24000
	else :
		fs=24
		fb="red"
		fc="white"
		spd=5.5
		sampleRate=22050

	#ffm='''ffmpeg -nostdin -loop 1 -i "{chartPath}" -i "{audioPath}" -shortest -tune stillimage -c:v libx264 -pix_fmt yuv420p -flags +global_header -c:a copy -movflags +faststart -vf "drawbox=y=ih-h-0:h=72:color={fb}@0.85:t=fill","subtitles={srtPath}:force_style='fontsize={fs},MarginV=5'" -y "{videoPath}"'''
	ffm='''ffmpeg -nostdin -loop 1 -i "{chartPath}" -i "{audioPath}" -shortest -tune stillimage -c:v libx264 -pix_fmt yuv420p -flags +global_header -c:a copy -movflags +faststart -vf "subtitles={srtPath}:force_style='fontsize={fs},MarginV=5'" -y "{videoPath}"'''
	xcmd=ffm.format(**locals())
	#return ffm
	try:
		p = Popen(xcmd,shell=True,bufsize=1024,stdout=PIPE,stdin=PIPE,stderr=PIPE)
		o, e = p.communicate()
	except Exception as e:
		sys.stderr.write("**ERROR: {} @ {}\n".format(str(e),'run_ffmpeg') )
		o=''
	if debugTF:
		sys.stderr.write("===SHELL_CMD:{}\n---OUT:{}\n---ERR:{}\n".format(xcmd,o,e))
	return o,e

def dat2rptTxt(ts='',dd={},lang='cn',mp3YN=False,ctrlSTRUCT=None,debugTF=False,_pn_={},**optx):
	rptTxt=''
	try:
		if ctrlSTRUCT is not None and ctrlSTRUCT in ['include','extends']:
			ts='{{% {} "{}" %}}'.format(ctrlSTRUCT,ts)
		rptTxt = jj_fmt(ts,dd,lang=lang,mp3YN=mp3YN,_pn_=_pn_,**optx)
		sys.stderr.write("=====dat2rptTxt: {}\n{}\n".format(rptTxt,optx) )
	except Exception as e:
		sys.stderr.write("**ERROR: {} @ {}\n".format(str(e),'dat2rptTxt') )
	return rptTxt

def dat2audioTxt(ts='',dd={},lang='cn',mp3YN=False,debugTF=False,_pn_={},**optx):
	return dat2rptTxt(ts=ts,dd=dd,lang=lang,mp3YN=mp3YN,debugTF=debugTF,_pn_=_pn_,**optx)

def dat2chart(datax={},title='',outdir='.',debugTF=False,**optx):
	chartPath="{}/{}.svg".format(outdir,title)
	return chartPath

def txt2audio(mp3comment='',audioPath='',lang='cn',tempo=1.25,debugTF=False,**optx):
	o,e = str2audio(mp3comment,fname=audioPath,lang=lang,tempo=tempo,debugTF=debugTF)
	return audioPath

def audio2srt(comment='',audioPath='',srtPath='',strTF=True,debugTF=False,lang='cn',**optx):
	ret=generateSubtitle(textFile=comment,audioFile=audioPath,srtFile=srtPath,lang=lang,strTF=strTF)
	return ret

def audio2video(audioPath='',videoPath='',srtPath='',chartPath='',datax={},lang='cn',debugTF=False,**optx):
	o,e = run_ffmpeg(audioPath=audioPath,videoPath=videoPath,srtPath=srtPath,chartPath=chartPath,lang=lang,debugTF=debugTF)
	return videoPath

def mp4_tst(ts='',category='EoD',title='daily_briefing',datax={},debugTF=False,**optx):
	ret = tmpl2mp4(ts=ts,category=category,title=title,debugTF=debugTF,**optx)
	return ret

def set_ht(_pn_):
	fx = lambda m,y: [m[x][y] for x in m['tmplLst'] if y in m[x]]
	ht = fx(_pn_,'ht3')
	if len(ht)<1:
		ht = fx(_pn_,'ht2')
	if len(ht)<1:
		ht = fx(_pn_,'ht1')
	headTitle='' if len(ht)<1 else ht[0]
	_pn_.update(headTitle=headTitle)
	return _pn_
	
def saveBriefing(_pn_,tablename="mkt_briefing_details",zpk=['category','templateName','rpt_time','lang'],jsonObjReloadTF=False):
	fx = lambda m,y: [m[x][y] for x in m['tmplLst'] if y in m[x] and m[x][y] is not None]
	try:
		if 'headTitle' not in _pn_:
			_pn_ = set_ht(_pn_)
		if 'comment' not in _pn_:
			ret = "\n".join(fx(_pn_,'comment'))
			_pn_.update(comment=ret)
		if 'mp3comment' not in _pn_:
			m3ret = "\n".join(fx(_pn_,'mp3comment'))
			_pn_.update(m3comment=m3ret)
		ret,_,emsg = upsert_mdb([_pn_],zpk=zpk,dbname='ara',tablename=tablename,ordered=False,jsonObjReloadTF=jsonObjReloadTF)
		sys.stderr.write("==errmsg:SAVE TO {}:{} {}\n".format(tablename,_pn_['pbdt'],str(emsg)))
	except Exception as e:
		sys.stderr.write("**ERROR:INSERT INTO {}: {} of _pn_:\n{}\n".format(tablename,str(e),_pn_))
	return _pn_

@saferun
def tmpl2mp4(ts='',category='ItD',title='intraday_briefing',datax={},debugTF=False,_pn_={},**optx):
	'''
	Create comment, mp3comment based on 'title' and svg and 
	save them into into MDB table: 'mkt_briefing_details' (for 'intraday_briefing' and 'daily_briefing')
	or 'daily_single_stock' (for title='daily_single_stock') 
	'''
	jsonObjReloadTF=False
	if title[-9:]=='_briefing':
		tablename="mkt_briefing_details"
		rptTxt = dat2rptTxt(ts=ts,_pn_=_pn_,debugTF=debugTF,**optx)
		zpk = ['category','templateName','rpt_time','lang']
		sys.stderr.write("==SAVE {} to MDB table:{}\n".format(title,tablename))
	elif title=='daily_single_stock':
		dbname='ara'
		tablename=title
		deltaTolerance= optx.pop('deltaTolerance',43200)
		ticker=getKeyVal(optx,'ticker','')
		o,m,e=find_mdb(dict(ticker=ticker),dbname=dbname,tablename=tablename,sortLst=['pbdt'])
		if len(o)>0 and 'pbdt' in o[0]:
			pbdtMod=Timestamp(o[0]['pbdt'])
			deltaPassed=int(pd.Timedelta(pd.datetime.now()-pbdtMod).total_seconds())
			if deltaPassed<=deltaTolerance:
				sys.stderr.write(" --pbdtMod:{},deltaPassed:{},deltaTolerance:{}\n".format(pbdtMod,deltaPassed,deltaTolerance))
				sys.stderr.write(" --Directly use comment from table: {}\n".format(tablename))
				return o[0]
		rptTxt = dat2rptTxt(ts=ts,_pn_=_pn_,debugTF=debugTF,**optx)
		_pn_['headTitle']=_pn_['stock_performance']['comment'];
		ticker=_pn_['ticker']=_pn_['stock_performance']['ticker'];
		_pn_['category']=category
		_pn_['templateName']=title
		_pn_['pbdt']=_pn_['stock_performance']['pbdt'];
		_pn_['lang']=getKeyVal(optx,'lang','cn')
		zpk=['ticker']
		jsonObjReloadTF=True
		sys.stderr.write("==SAVE {}:{} to MDB table:{}\n".format(title,ticker,tablename))
	else:
		return {}
	_pn_.update(title=title)
	saveBriefing(_pn_,tablename=tablename,zpk=zpk,jsonObjReloadTF=jsonObjReloadTF)
	if debugTF:
		sys.stderr.write("=====ts:\n{}\n".format(ts))
		sys.stderr.write("=====optx:\n{}\n".format(optx))
		sys.stderr.write("=====rptTxt:\n{}\n".format(rptTxt))
		sys.stderr.write("=====_pn_:\n{}\n".format(_pn_))
		sys.stderr.write("=====keys:\n{}\n".format(_pn_.keys()))
	if 'tmplLst' in _pn_:
		tmplLst = _pn_['tmplLst']
		sys.stderr.write("=====tmplLst: {}\n".format(tmplLst))
		for tname in tmplLst:
			dx = _pn_[tname]
			sys.stderr.write("===Chartpath: {}:{}\n".format(tname,dx['chartpath']))
	else:
		sys.stderr.write("*ERROR:tmplLst not found in {}\n".format(_pn_.keys()))
		return {}
	txtOnly=optx.pop('txtOnly',False)
	if txtOnly is True:
		sys.stderr.write("=====RUN Text Only: {}\n".format(txtOnly))
		return _pn_
	sys.stderr.write("=====RUN pn2mp4: {}\n".format(tmplLst))
	dpn = pn2mp4(_pn_=_pn_,zpk=zpk,**optx)
	return dpn

def pn2mp4(_pn_={},zpk=[],debugTF=False,**optx):
	'''
	convert _pn_ that contail svg, comment, mp3comment into relevant mp3 and mp4 files
	and save file locations and save to MDB table: 'mkt_briefing_media' or 'daily_single_stock_media' 
	'''
	if 'tmplLst' not in _pn_:
		return {}
	tmplLst = _pn_['tmplLst']
	title = _pn_['title']
	dpn=subDict(_pn_,zpk+['title','pbdt','headTitle','intraday_headline','daily_headline','comment','mp3comment'])
	videoLst=[]
	j=0
	for tname in tmplLst:
		if tname not in _pn_:
			sys.stderr.write("===NotFound: @ {}\n".format(tname))
			continue
		sys.stderr.write("===Running: {}\n{}\n".format(tname,_pn_[tname]))
		try:
			dx = _pn_[tname]
			if 'chartpath' not in dx or dx['chartpath'] is None:
				continue
			chartpath= dx['chartpath']
			xtmp=re.sub("(.svg)?","",chartpath)
			outdir,mpname=os.path.dirname(xtmp),os.path.basename(xtmp)
			if len(mpname)<1:
				sys.stderr.write("**WARNING: @ {}\t{}\n".format(tname,"Filename not exists"))
				continue
			dx.pop('title',None)
			vk = [k for k in dx]
			for k in vk:
				v = dx[k]
				if isinstance(v,(dict,tuple,list,pd.DataFrame)):
					dx.pop(k,None)
					#dx.update(k=None)
			txtChart2audioSrtVideo(tmplname=tname,mpname=mpname,tmplobj=dx,debugTF=debugTF,**optx)
			dpn.update({tname:dx})
			videoLst.append(dx['videoPath'])
			sys.stderr.write("\n==={}:{} successfully created!\n".format(tname,dx['videoPath']))
		except Exception as e:
			sys.stderr.write("**ERROR: @ {}\n\t{} @{}\n".format(tname,str(e),'pn2mp4'))
			continue
		j=j+1
	if j<1:
		return {}
	sys.stderr.write("===Total:{}: {}\n".format(j,videoLst))
	try:
		xpoch = mpname.split("_")[-1]
		if title=='daily_single_stock':
			ticker = getKeyVal(_pn_,'ticker',None)
			videoPath='{}_{}_{}_ALL.mp4'.format(title,ticker,xpoch)
		else:
			videoPath='{}_{}_ALL.mp4'.format(title,xpoch)
		run_concat(videoLst=videoLst,outdir=outdir,videoPath=videoPath)
		#dpn.update(comment=rptTxt)
		dpn.update(videoPath=videoPath)
		sys.stderr.write("===Combine: {} To {}\n".format(videoLst,videoPath))
		if len(glob(outdir+"/"+videoPath))>0:
			sys.stderr.write("===videoLst: {}\n".format(videoLst))
			sys.stderr.write("===videopath: {} successfully created!\n".format(videoPath))
		else:
			sys.stderr.write("**ERROR: {} not created!\n".format(videoPath))

		# save to MDB 'dbname'::'tablename'
		dbname=optx.pop('dbname','ara')
		tablename=optx.pop('tablename','')
		if title[-9:]=='_briefing':
			tablename='mkt_briefing_media'
		elif title=='daily_single_stock':
			tablename=title+'_media'
		if len(tablename)>5:
			ret,_,emsg = upsert_mdb([dpn],zpk=zpk,dbname=dbname,tablename=tablename,ordered=False)
			sys.stderr.write("==errmsg:SAVE TO {} @ {}, STATUS:{}\n".format(tablename,dpn['pbdt'],str(emsg)))
	
		sys.stderr.write("==SUCCESS videoLst:{}, mp4:{}\n".format(videoLst,mpname))
	except Exception as e:
		sys.stderr.write("**ERROR: @ {}:{}\n\t{}\n".format(videoLst,mpname,str(e)))
	return dpn

def txtChart2audioSrtVideo(tmplname='EoD',mpname='',tmplobj={},debugTF=False,**optx):
	'''
	Create srt, mp3 & mp4 given 'rptTxt', 'audioTxt' and 'chartpath'
	and save them in 'srtPath','audioPath' and 'videoPath' respectively
	'''
	vTF=[x in tmplobj for x in ['chartpath','comment','mp3comment']]
	if any(vTF) is False:
		return {}
	outdir=getKeyVal(optx,'outdir','.')
	audioPath="{}/{}.mp3".format(outdir,mpname)
	videoPath="{}/{}.mp4".format(outdir,mpname)
	srtPath="{}/{}.srt".format(outdir,mpname)
	rptTxt = tmplobj['comment']
	chartPath = tmplobj['chartpath']
	audioTxt = tmplobj['mp3comment']
	audioPath = txt2audio(mp3comment=audioTxt,audioPath=audioPath,debugTF=debugTF,**optx)
	srtTxt = audio2srt(comment=rptTxt,audioPath=audioPath,srtPath=srtPath,strTF=True,debugTF=debugTF,**optx)
	videoPath = audio2video(videoPath=videoPath,audioPath=audioPath,srtPath=srtPath,chartPath=chartPath,debugTF=debugTF,**optx)
	tmplobj.update(tmplname=tmplname,title=mpname,rptTxt=rptTxt,audioTxt=audioTxt,audioPath=audioPath,srtPath=srtPath,chartPath=chartPath,videoPath=videoPath)
	return tmplobj

def find_file(fn):
	if isinstance(fn,list):
		for f in fn:
			ret = glob(f)
			if len(ret)>0:
				return ret
		return []
	else:
		return glob(fn)
	
def tmpl_wrapper(tmplname,debugTF=False,_pn_={},tmplLst=[],lang='cn',ctrlSTRUCT='include',outdir='US/mp3_hourly',dirname='templates',**optx):
	fn="{dirname}/{tmplname}_cn.j2\n{dirname}/{tmplname}.j2".format(dirname=dirname,tmplname=tmplname).split("\n")
	tLst = find_file(fn)
	if len(tLst)<1:
		return {}
	ts = os.path.basename(tLst[0])
	dpn = tmpl2mp4(ts=ts,title=tmplname,debugTF=debugTF,_pn_=_pn_,tmplLst=tmplLst,lang=lang,ctrlSTRUCT=ctrlSTRUCT,outdir=outdir,dirname=dirname,**optx)
	return dpn

from _alan_optparse import parse_opt
def main_tst(description="Stock Update MP4 Creator",usage="Usage: %prog [option] daily_briefing|intraday_briefing [SYMBOL] [True|False]"):
	opts, args = parse_opt(sys.argv, description=description,usage=usage)
	ticker=getKeyVal(opts,'ticker','AAPL')
	txtOnly=getKeyVal(opts,'txtOnly',False)
	debugTF=getKeyVal(opts,'debugTF',True)
	deltaTolerance=getKeyVal(opts,'deltaTolerance',43200)
	_pn_={}
	tmplname = 'daily_briefing' if len(args)<1 else args[0]
	ticker = ticker if len(args)<2 else args[1]
	txtOnly = True if len(args)>2 and args[2][:1].lower() in ['1','t'] else txtOnly
	sys.stderr.write("===RUN:{}:{}:{}\n".format(tmplname,ticker,txtOnly))
	dpn = tmpl_wrapper(tmplname,ticker=ticker,debugTF=debugTF,txtOnly=txtOnly,deltaTolerance=deltaTolerance)
	return dpn


if __name__ == '__main__':
	main_tst()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''
Get [CIK,SIC,Company] info from [TICKER] via
http://www.sec.gov/cgi-bin/browse-edgar?CIK={ticker}&Find=Search&owner=exclude&action=getcompany&output=atom
Get GIC [Sector] via
https://query2.finance.yahoo.com/v11/finance/quoteSummary/{ticker}?modules=summaryProfile'
Convert chinese [company_cn] via tencent site: http://qt.gtimg.cn/q=usAAPL
http://qt.gtimg.cn/q=us{ticker}

Using feedparser
[feed] keys:
['addresses', 'links', 'address', 'updated', 'assigned-sic', 'id', 'updated_parsed', 'city', 'zip', 'author', 'title_detail', 'state', 'state-of-incorporation', 'conformed-name', 'state-location-href', 'street1', 'assigned-sic-desc', 'phone', 'assigned-sic-href', 'link', 'authors', 'author_detail', 'ticker_sec', 'cik-href', 'fiscal-year-end', 'cik', 'title', 'guidislink', 'company-info', 'assitant-director', 'state-location']

Last mod., Wed Jul 17 10:07:29 EDT 2019
by Ted
'''

import sys
import requests
import feedparser
import pandas as pd
from sqlalchemy import create_engine
if sys.version_info.major == 2:
	reload(sys)
	sys.setdefaultencoding('utf8')
headers={'Content-Type': 'text/html', 'Accept': 'application/json', 'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36'}


def subDict(M,K=[],reverseTF=False):
	"""
	return a subset of M that matches K keys
	OR
	if reverse is True
	a subset of M that does not match K keys
	"""
	if reverseTF is True: # invert-match, select non-matching [kyLst] keys
		return { ky:va for ky,va in M.items() if ky not in K }
	else:
		return { ky:va for ky,va in M.items() if ky in K }

# ticker to get [CIK,SIC,Company] info via sec-edgar site
def get_company_info(ticker):
	'''
	Get [CIK,SIC,Company] info from [TICKER] via
	http://www.sec.gov/cgi-bin/browse-edgar?CIK={ticker}&Find=Search&owner=exclude&action=getcompany&output=atom
	'''
	tkX=ticker.upper().replace('-','').replace('.','')
	urx = 'http://www.sec.gov/cgi-bin/browse-edgar?CIK={}&Find=Search&owner=exclude&action=getcompany&output=atom'
	url = urx.format(tkX)
	d = feedparser.parse(url)
	if 'cik' in d.feed:
		d.feed['ticker_sec'] = tkX
		d.feed['ticker'] = ticker
	return d

def parse_feed(f):
	vky=dict(ticker="ticker",ticker_sec="ticker_sec",cik="cik",
		company="conformed-name",sic="assigned-sic",yymm="fiscal-year-end")
	a={}
	for (j,(xk,xv)) in enumerate(vky.items()) :
		if f.has_key(xv):
			a[xk]=f[xv]
		else:
			a[xk]=''
		if xk in ["cik","sic","yymm"]:
			a[xk]=int(a[xk]) if len(a[xk])>0 else None
		elif xk in ["company"]:
			a[xk]=a[xk].upper()
	return a

def ticker2cik(ticker):
	tkX=ticker.upper()
	d = get_company_info(tkX)
	if d.feed.has_key('cik') is False :
		return {}
	f = parse_feed(d.feed)
	return f

def ticker_mapping_batch(tkLst,debugTF=False):
	vf=[]
	for tkX in tkLst:
		f=fst=fcn={}
		# GET cik, sic info
		try:
			f = ticker2cik(tkX)
			if not f:
				f.update(cik=-999999,sic=-999,yymm=1231,ticker_sec=tkX)	
		except Exception as e:
			errmsg="**ERROR: {} of {} @ {}\n".format(str(e),tkX,'ticker_mapping_batch')
			sys.stderr.write(errmsg)
			f.update(cik=-999999,sic=-999,yymm=1231,ticker_sec=tkX)	
			#continue
		# GET sector info
		try:
			fst = ticker2sector(tkX)
			if fst:
				fst['sector_alias'] = fst['sector']
				f.update(subDict(fst,['sector_alias','industry']))
			else:
				f['sector_alias'] = f['industry'] = ''
		except Exception as e:
			errmsg="**ERROR: {} of {} @ {}\n".format(str(e),tkX,'running ticker2sector')
			sys.stderr.write(errmsg)
		# GET company_cn info
		try:
			fcn = ticker2cn(tkX)
			if fcn:
				f.update(fcn)
			else:
				f['company_cn'] = txt2tw(f['company'])
		except Exception as e:
			errmsg="**ERROR: {} of {} @ {}\n".format(str(e),tkX,'running ticker2cn')
			sys.stderr.write(errmsg)
			continue
		if f:
			sys.stderr.write("{}\n".format(f))
			vf.append(f)
		
	return vf

def write2db(df,engine=None,dbname='ara',tablename='',rmode='replace'):
	if not tablename:
		return df
	if not engine:
		engine = create_engine('postgresql://sfdbo@localhost:5432/'+dbname)
	df.to_sql(tablename,engine,index=False,schema='public',if_exists=rmode)
	engine.dispose()
	return df

def txt2twOLD(s,dest='zh-tw'):
	from googletrans import Translator
	tranx=Translator().translate
	return tranx(s,dest=dest).text

def txt2tw(dscr='',src='en',dest='zh-TW'):
	from google_trans_new import google_translator as Translator
	import html
	dscr =html.unescape(dscr)
	dscr = str(dscr.strip()+"\n")
	if len(dscr)<1:
		return dscr
	try:
		newd = Translator().translate(dscr,lang_src=src,lang_tgt=dest)
		return newd
	except Exception as e:
		sys.stderr.write("**ERROR: @{} of {}\nINPUT:\n{}\n".format("txt2tw",str(e),dscr))
		return ''

# ticker to get chinese 'company_cn' via http://qt.gtimg.cn
def ticker2cn(ticker,urx='http://qt.gtimg.cn/q=us{}',debugTF=False):
	''' 
	To get company name in chinese based on ticker 'ticker'
	Return dict {ticker,company,company_cn}
	'''
	r=requests.get(urx.format(ticker.upper()))
	d = r.text.split('~')
	if len(d)<2:
		return {}
	company_cn = d[1]
	company = d[46]
	if debugTF:
		sys.stderr.write("{}|{}\n".format(ticker,company_cn))
	try:
		company_cn = txt2tw(company_cn)
	except Exception as e:
		sys.stderr.write("**ERROR: {} @ {}\n".format(str(e),"ticker2cn"))
	da = dict(zip(("ticker","company_cn","company"),(ticker,company_cn,company)))
	return da

# ticker to get 'sector' via yahoo finance site
def ticker2sector(ticker,urx='',debugTF=False):
	'''
	Get GIC [Sector] via
	https://query2.finance.yahoo.com/v11/finance/quoteSummary/{ticker}?modules=summaryProfile'
	'''
	if not urx:
		urx='https://query2.finance.yahoo.com/v11/finance/quoteSummary/{ticker}?modules=summaryProfile'
	try:
		r = requests.Session().get(urx.format(ticker=ticker),headers=headers)
		js = r.json()
		jd = js["quoteSummary"]["result"][0]['summaryProfile']
	except Exception as e:
		sys.stderr.write("**ERROR: {} @ {}\n".format(str(e),'ticker2sector'))
		return {}
	return jd

def ticker_mapping(tkLst,saveDB=False):
	try:
		dd = ticker_mapping_batch(tkLst)
		if len(dd)<1:
			return {}
		df = pd.DataFrame(dd)
		write2db(df,tablename='ticker_mapping_temp')
	except Exception as e:
		sys.stderr.write("**ERROR: {}\n".format(str(e)))
		return {}
	return df

if __name__ == '__main__':
	args=sys.argv[1:]
	if len(args)<1 or args[0]=='-':
		tkLst = sys.stdin.read().split()
		saveDB=True
	else:
		tkLst=args
		saveDB=False
	dd = ticker_mapping(tkLst,saveDB=saveDB)
	if len(dd)>0:
		sys.stdout.write("{}\n".format(dd.to_csv(index=False,sep='|')))



#!/usr/bin/env python 
"""
Program: _alan_date.py
Description:
	Date functions for ALAN,
	convert daily price into monthly and update to prc_m_hist
Functions:
	def s2dt(s=None,dformat=''):
	def get_start_end(start=None,end=None,dformat='',**kwargs):
	def str2epoch(s=None,dformat=''):
	def next_date(d=None,dformat='',dtTF=True,**kwargs):
	def next_month_date(d=None,months=1):
	def delta2dates(end,start,dformat="%Y%m%d",fq="D",rounding=0):
	def delta2datesOLD(end,start,dformat="%Y%m%d",fq="D",rounding=0):
	def ymd_diff(start,end,dformat="%Y%m%d"):
	def epoch_parser(x,s=1000): return datetime.fromtimestamp(int(x/s))
	def ymd_parser(x,fmt='%Y%m%d'): return datetime.strptime(str(x),fmt)
	def ymd2dt(s,dformat="%Y%m%d"):
	def ymd_delta(ymd=None,days=0,dformat="%Y%m%d"):
	def ymd_delta_OLD(ymd=None,days=0,dformat="%Y%m%d"):
	def dt2ymd(dt,dformat="%Y%m%d"):
	def ymd2dt_example():
	def dt2ymd_example():
	def freq_d2m(s,fq='M',method='last'):
	def daily2month(fromSql=None,tablename=None,dbname="ara",hostname="localhost",wmode='fail',fq='M',method='last'):
	def tg_next2week(pdt,cdt=None,nwk=1):
	def tg_latest2week(pdt,cdt=None,nwk=1):
Last mod., Tue Apr 30 10:32:36 EDT 2019
Version: 0.70
"""
import sys
import numpy as np
from math import isinf, log
from datetime import datetime,timedelta,date
from dateutil import relativedelta
from _alan_calc import subDict

def seOfDay(d, startOfDay=False, endOfDay=False):
	"""
	return datetime of the start/end of a day on the date 'd'
	"""
	if startOfDay is True:
		d = datetime.strptime(d.strftime('%Y%m%d'),'%Y%m%d')
	elif endOfDay is True:
		d = datetime.strptime(d.strftime('%Y%m%d'),'%Y%m%d') + timedelta(days=1) -  timedelta(seconds=1)
	return d

def s2dt(s=None,dformat='',startOfDay=False,endOfDay=False):
	"""
	convert any of [Ymd,Y-m-d,Y/m/d,epoch,epochUnix] to datetime
	or any string with date format 'dformat`
	return None if s is not a string or None
	"""
	if isinstance(s,date):
		return seOfDay(s,startOfDay=startOfDay,endOfDay=endOfDay)
	#elif any([True for x in (None,[],{},(),0,range(0),'') if x==s]):
	#elif s is None or (hasattr(s,'__len__') and len(s)<1):
	elif not s:
		d = datetime.now()
		return seOfDay(d,startOfDay=startOfDay,endOfDay=endOfDay)
	elif isinstance(s,float) and not isinf(s):
		s = str(int(s))
	elif isinstance(s,(int,np.integer)):
		s = str(s)
	elif not isinstance(s,str):
		return None
	if isinstance(dformat,str) and len(dformat)>1:
		d = datetime.strptime(s,dformat)
	elif len(s)>=10 and s.isdigit():
		d = datetime.fromtimestamp(int(s[:10]))
	elif len(s)==8 and s.isdigit():
		d = datetime.strptime(s,"%Y%m%d")
	elif len(s)==10 and '-' in s:
		d = datetime.strptime(s,"%Y-%m-%d")
	elif len(s)==10 and '/' in s:
		d = datetime.strptime(s,"%Y/%m/%d")
	else:
		d = datetime.now()
	return seOfDay(d,startOfDay=startOfDay,endOfDay=endOfDay)

def get_start_end(start=None,end=None,dformat='',startOfDay=False,endOfDay=False,**kwargs):
	"""
	return end and start dates, based on 'end', 'start' strings
	where
	'start' and 'end' can be any of [Ymd,Y-m-d,Y/m/d,epoch,epochUnix,None]
	Default to current datetime if 'end' is None,
	Default to 'end' - 'kwargs' options if 'start' is None 
	and only 'end' is applied if 'kwargs' is empty. 
	"""
	end=s2dt(end,dformat,startOfDay=startOfDay,endOfDay=endOfDay)
	if start is None:
		start = end	
		if kwargs:
			try:
				start -= relativedelta.relativedelta(**kwargs) 
			except Exception as e:
				sys.stderr.write("**ERROR regs relativedelta: {}\n".format(str(e)))
	else:
		start=s2dt(start,dformat,startOfDay=startOfDay,endOfDay=endOfDay)
	return (start,end)

def str2epoch(s=None,dformat='',startOfDay=False,endOfDay=False):
	"""
	return string of [Ymd,Y-m-d,Y/m/d,epoch,epochUnix] to epoch in integer
	"""
	e=s2dt(s,dformat,startOfDay=startOfDay,endOfDay=endOfDay)
	return e if e is None else int(e.strftime("%s"))

def next_date(d=None,dformat='',dtTF=True,startOfDay=False,endOfDay=False,**kwargs):
	""" return datetime w.r.t. datetime 'd' based on options 'kwargs'
	Where
	kwargs can be years, months, weeks, days, hours, minutes, seconds, microseconds
	as the delta difference from current 'd'
	and month (of the year), weekday (forthcoming, Monday:0), day (of the month)
	e.g.,
	# return 1st date of the last month from current datetime
	next_date(None,months=-1,day=1)
	# return 1st date of March from current datetime (with same year)
	next_date(None,month=3)
	# return last month from current datetime (with same year and day)
	next_date(None,dformat='%Y%m%d',months=-1,dtTF=False)
	# return last date of Feburary from current datetime (with same year)
	next_date(None,month=2,day=31)
	"""
	dt = s2dt(d,dformat)
	if len(kwargs)>0:
		dt = dt + relativedelta.relativedelta(**kwargs)
	dt = s2dt(dt,startOfDay=startOfDay,endOfDay=endOfDay)
	if len(dformat)>1 and dtTF is False:
		return dt.strftime(dformat)
	else:
		return dt

def next_month_date(d=None,months=1):
	""" return same datetime [d] for the next month """
	if not d:
		d = datetime.now()
	return d + relativedelta.relativedelta(months=months)

def delta2dates(end,start,dformat="%Y%m%d",fq="D",rounding=0):
	""" return difference of 2 dates, end - start in day, month or year
	""" 
	if fq.upper()=="M":
		md=relativedelta.relativedelta(ymd2dt(end,dformat),ymd2dt(start,dformat))
		xdif=md.years*12+md.months
		if rounding > 0:
			xsign = -1 if md.days < 0 else 1
			xdif=xdif+int((md.days+30*xsign)/31)
	elif fq.upper()=="Y":
		md=relativedelta.relativedelta(ymd2dt(end,dformat),ymd2dt(start,dformat))
		xdif=md.years
		if rounding == 1:
			xsign = -1 if md.days < 0 else 1
			xdif=xdif+int(md.months+int((md.days+30*xsign)/31) + 11*xsign)/12
	elif fq.upper()=="W":
		md=ymd2dt(end,dformat)-ymd2dt(start,dformat)
		if rounding == 1:
			xsign = -1 if md.days < 0 else 1
			xdif=int((md.days+6*xsign)/7)
		else:
			xdif=int(md.days/7)
	elif fq.upper()=="HOUR":
		md=ymd2dt(end,dformat)-ymd2dt(start,dformat)
		if rounding == 1:
			xsign = -1 if md.total_seconds() < 0 else 1
			xdif=int((md.total_seconds()+3540*xsign)/3600)
		else:
			xdif=int(md.total_seconds()/3600)
	elif fq.upper()=="MINUTE":
		md=ymd2dt(end,dformat)-ymd2dt(start,dformat)
		if rounding == 1:
			xsign = -1 if md.total_seconds() < 0 else 1
			xdif=int((md.total_seconds()+59*xsign)/60)
		else:
			xdif=int(md.total_seconds()/60)
	elif fq.upper()=="SECOND":
		md=ymd2dt(end,dformat)-ymd2dt(start,dformat)
		xdif=md.seconds
	else:
		md=ymd2dt(end,dformat)-ymd2dt(start,dformat)
		xdif=md.days
	return xdif

def delta2datesOLD(end,start,dformat="%Y%m%d",fq="D",rounding=0):
	""" return difference of 2 dates, end - start in day, month or year
	""" 
	if fq.upper()=="M":
		md=relativedelta.relativedelta(datetime.strptime(str(end),dformat),datetime.strptime(str(start),dformat))
		xdif=md.years*12+md.months
		if rounding == 1:
			xdif=xdif+int((md.days+30)/31)
	elif fq.upper()=="Y":
		md=relativedelta.relativedelta(datetime.strptime(str(end),dformat),datetime.strptime(str(start),dformat))
		xdif=md.years
		if rounding == 1:
			xdif=xdif+int(md.months+int((md.days+30)/31) + 11)/12
	elif fq.upper()=="W":
		md=datetime.strptime(str(end),dformat) - datetime.strptime(str(start),dformat)
		if rounding == 1:
			xdif=int((md.days+6)/7)
		else:
			xdif=int(md.days/7)
	else:
		md=datetime.strptime(str(end),dformat) - datetime.strptime(str(start),dformat)
		xdif=md.days
	return xdif

def ymd_diff(start,end,dformat="%Y%m%d"):
	""" return difference in days of end-start
	"""
	xdif=datetime.strptime(str(end),dformat) - datetime.strptime(str(start),dformat)
	return xdif.days

def epoch_parser(x,s=1000): return datetime.fromtimestamp(int(x/s)) if x is not None else x

def ymd_parser(x,fmt='%Y%m%d'): return datetime.strptime(str(x),fmt) if x is not None else x

def ymd2dt(s,dformat="%Y%m%d"):
	"""
	convert [s] into a datetime struct format based on format: [dformat]
	note:
	  1. s can be a string or a list of string
	  2. s is treated as a epoch number if s is digit and s >= 10 digits

	"""
	if isinstance(s,(list,tuple,np.ndarray)):
		return [ymd2dt(x,dformat=dformat) for x in s]
	if s is None or isinstance(s,date):
		return s
	elif (isinstance(s,(np.integer,int,float)) or s.isdigit() ): 
		s = int(s) 
		if log(s,10) >= 12:
			s = s/1000
		if log(s,10) >= 9:
			return datetime.fromtimestamp(s)
	return datetime.strptime(str(s),dformat)

def ymd_delta(ymd=None,days=0,dformat="%Y%m%d"):
	""" return date of past [days] from [ymd] date in [dformat] format
	"""
	if ymd is not None and len(str(ymd))>4 and dformat is not None and len(dformat)>1:
		dt=datetime.strptime(str(ymd),dformat)
	elif isinstance(ymd,datetime):
		dt=ymd
	else:
		dt=datetime.now()
	if days != 0:
		dt=dt-timedelta(days=days)
	if dformat is None or len(dformat)<2:
		ret = dt
	else:
		ret = datetime.strftime(dt,dformat)
	return ret

def ymd_delta_OLD(ymd=None,days=0,dformat="%Y%m%d"):
	""" return date in format: [dformat] of [days] days from [ymd]
	"""
	if ymd is None:
		dt=datetime.now()-timedelta(days=days)
	else:
		dt=datetime.strptime(str(ymd),dformat)-timedelta(days=days)
	return datetime.strftime(dt,dformat)

def dt2ymd(dt,dformat="%Y%m%d"):
	"""
	Convert datetime 'dt' to struct format 'dformat' 
	Note that s can be a string or a list of string
	"""
	if isinstance(dt,(list,tuple,np.ndarray)):
		return [dt2ymd(x,dformat) for x in dt]
	if isinstance(dt,date):
		return dt.strftime(dformat)
	elif not dt:
		return datetime.strftime(datetime.now(),dformat)
	else:
		return dt

def ymd2dt_example():
	ymd=dt2ymd(datetime.today())
	return 'ymd2dt({})={}'.format(ymd,ymd2dt(ymd))

def dt2ymd_example():
	return 'dt2ymd(today)={}'.format(dt2ymd(datetime.now()))

def freq_d2m(s,fq='M',method='last',dtcol='pbdate',debugTF=False):
	'''
	Convert pandas.Series 's' to new frequency 'fq' of method 'method'
	Note dtcol is used if s.index is not DatetimeIndex
	
	'''
	import pandas as pd
	freq = fq[:1].upper()
	mth = method.lower()
	if not isinstance(s,pd.DatetimeIndex):
		s.index= [ ymd2dt(x) for x in s[dtcol] ]
	try:
		ml = getattr(s.resample(freq),mth)()
	except Exception as e:
		sys.stderr.write("**ERROR:{}:{}\n".format(mth,e))
	if debugTF:
		sys.stderr.write("Method: {}\n".format(mth))
		sys.stderr.write("{}\n".format(ml.tail()))
	return ml

def daily2month(fromSql=None,tablename=None,dbname="ara",hostname="localhost",wmode='fail',fq='M',method='last',output=''):
	""" Get price history using yahoo and save them into 'prc_temp_yh'
	"""
	from sqlalchemy import create_engine
	import pandas as pd
	if not fromSql: 
		fromSql="SELECT * FROM prc_hist WHERE name='IBM' ORDER BY pbdate"
	if tablename is None:
		tablename="spdr_price_m_hist"
	# Connect to DB
	dbURL='postgresql://sfdbo@{0}:5432/{1}'.format(hostname,dbname)
	engine = create_engine(dbURL)
	# Get stock price history
	sDu = pd.read_sql(fromSql,con=engine) 

	# Get stock frequency conversion
	tsp = freq_d2m(sDu,method=method,fq=fq)

	# Save stock price history to {tablename}
	if wmode in ['replace','append'] :
		tsp.to_sql(tablename,engine,schema='public',index=False,if_exists=wmode)
	if output=='csv':
		sys.stdout.write(tsp.to_csv(index=False,sep="\t")+"\n")
	# Close DB
	engine.dispose()
	return tsp

def tg_next2week(pdt,cdt=None,nwk=1):
	"""
	check of target date 'pdt' is within up-to-next week Friday range
	"""
	if cdt is None:
		cdt = next_date()
	if pdt<cdt and pdt.day!=cdt.day:
		return {}
	gap = nwk
	xdt = next_date(cdt,weeks=gap,weekday=4) # monday of the last week
	sundt = next_date(xdt,days=-5) # coming sunday from xdt(monday)
	if pdt>xdt: # discard any data bigger than Friday of the next week
		sys.stderr.write("Too far to report:{} > {}, disregard!\n".format(pdt,xdt))
		return {}
	currWeekTF= (pdt<sundt)
	todayTF= (pdt.day==cdt.day)
	d=dict(currWeekTF=currWeekTF,todayTF=todayTF,endDate=xdt,currDate=cdt,tgDate=pdt)
	d.update(tgWeekday=pdt.weekday()+1)
	return d

def tg_latest2week(pdt,cdt=None,nwk=1):
	"""
	check of target date 'pdt' is within last week's Monday range
	"""
	if cdt is None:
		cdt = next_date()
	if pdt>cdt and pdt.day!=cdt.day:
		return {}
	gap = -nwk if cdt.weekday()==0 else -(nwk+1)
	xdt = next_date(cdt,weeks=gap,weekday=0) # monday of the last week
	sundt = next_date(xdt,days=5) # coming sunday from xdt(monday)
	if pdt<=xdt: # discard any data less Monday of the week
		sys.stderr.write("Stale news:{} < {}, disregard!\n".format(pdt,xdt))
		return {}
	currWeekTF= (pdt>sundt)
	todayTF= (pdt.day==cdt.day)
	d=dict(currWeekTF=currWeekTF,todayTF=todayTF,startDate=xdt,currDate=cdt,tgDate=pdt)
	d.update(tgWeekday=pdt.weekday()+1)
	return d

def utc2local(s,tz1='UTC',tz2='US/Eastern',fmt="%b-%d %I:%M%p %Z"):
	''' convert datetime-string from `tz1` to `tz2` in format `fmt` 
	'''
	import pandas as pd
	return pd.Timestamp(s,tz=tz1).tz_convert(tz=tz2).strftime(fmt)

if __name__ == '__main__':
	fromSql="SELECT * FROM spdr_price_hist WHERE ticker='{}' ORDER BY pbdate"
	ticker=sys.argv[1] if len(sys.argv)>1 else "IBM"
	fromSql=fromSql.format(ticker)

	sys.stderr.write("{}\n".format(daily2month(fromSql).tail()))
	print("20180221-20170320 in months:",delta2dates(20180221,20170320,fq="M"))



#!/usr/bin/env python3
'''
Get daily/minute data via yahoo version 8/11 api
Usage of:
yh_hist_batch.py IBM --output=csv --range=1d
OR (To apply 'yh_hchg' as function argument )
yh_hist_batch.py ^GSPC --output=csv --range=5d --gap=1d --extra_xs='funcArg="yh_hchg"
OR
python3 -c "from yh_hist_batch import yh_hist; dx=yh_hist(ticker='^GSPC',gap='1m',ranged='5m');print(dx.to_json(orient='records'))"
OR
python3 -c "from yh_hist_batch import yh_hchg; dx=yh_hchg(ticker='^GSPC',gap='1m',ranged='5m');print(dx.to_json(orient='records'))"

Ref site:
# 1-minute data for 1-day (use indicators=close for selecting columns like IEX filter)
# with validRanges:["1d","5d","1mo","3mo","6mo","1y","2y","5y","10y","ytd","max"]
https://query1.finance.yahoo.com/v8/finance/chart/AAPL?region=US&lang=en-US&includePrePost=false&interval=1m&range=1d

# 1-minute data for 1-day (NOTE, period1,period2 need to be a valid data date due to yahoo's setup) 
# with [start,end] periods:[period1,period2] in epochTime
https://query1.finance.yahoo.com/v8/finance/chart/AAPL?region=US&lang=en-US&includePrePost=false&interval=1m&period1=1553832000&period2=1553918400

# daily data for 1-day
# with validRanges:["1d","5d","1mo","3mo","6mo","1y","2y","5y","10y","ytd","max"]
https://query1.finance.yahoo.com/v8/finance/chart/AAPL?region=US&lang=en-US&includePrePost=false&interval=1d

# multi-ticker 5-minute data for 1-day (use indicators=close for selecting columns like IEX filter)
https://query1.finance.yahoo.com/v7/finance/spark?symbols=AAPL,IBM&range=1d&interval=5m&indicators=close&includeTimestamps=false&includePrePost=false&corsDomain=finance.yahoo.com&.tsrc=finance

# multi-ticker data onTheFly quote 
https://query1.finance.yahoo.com/v7/finance/quote?symbols=AAPL,IBM
https://query1.finance.yahoo.com/v7/finance/quote?fields=symbol,longName,regularMarketPrice,&formatted=false&symbols=MMM,JNJ

# options data (for ticker: amd)
https://query1.finance.yahoo.com/v7/finance/options/amd

# historical daily data
# with [start,end] periods:[period1,period2] in epochTime
https://query1.finance.yahoo.com/v8/finance/chart/AAPL?region=US&lang=en-US&includePrePost=false&interval=1d&period1=1551416400&period2=1554350400

# financial data
https://query2.finance.yahoo.com/v11/finance/quoteSummary/AAPL?modules=summaryProfile,financialData,defaultKeyStatistics
# where module:
assetProfile,balanceSheetHistory,balanceSheetHistoryQuarterly,calendarEvents,cashflowStatementHistory,cashflowStatementHistoryQuarterly,defaultKeyStatistics,earnings,earningsHistory,earningsTrend,financialData,fundOwnership,incomeStatementHistory,incomeStatementHistoryQuarterly,indexTrend,industryTrend,insiderHolders,insiderTransactions,institutionOwnership,majorDirectHolders,majorHoldersBreakdown,netSharePurchaseActivity,price,quoteType,recommendationTrend,secFilings,sectorTrend,summaryDetail,summaryProfile,symbol,upgradeDowngradeHistory,fundProfile,topHoldings,fundPerformance

Last Mod., Wed Apr  3 17:42:40 EDT 2019
'''

#'^GSPC','^TWII','000001.SS','^SOX','^DJI'
import sys
from optparse import OptionParser
import pandas as pd
import numpy as np
from datetime import datetime,timedelta
import pytz
import requests
from _alan_calc import pqint,getKeyVal,subDict,subDF

def epoch_parser(x,s=1000): return datetime.fromtimestamp(int(x/s))
def ymd_parser(x,fmt='%Y%m%d'): return datetime.strptime(str(x),fmt)

def eten_hist(ticker=None,gap='1m',ranged='1d',hdrTF=True,tsTF=True,debugTF=False):
	if gap == '1d':
		from _alan_calc import sqlQuery
		xqr="SELECT * FROM prc_hist WHERE name='{}' AND pbdate>20171101".format(ticker)
		return sqlQuery(xqr,dbname='ara.tw')
	twtime = datetime.now(pytz.timezone('Asia/Taipei'))
	if twtime.hour < 9:
		pbdate = (twtime - timedelta(days=1)).strftime('%Y%m%d')
	else:
		pbdate = (twtime).strftime('%Y%m%d')
	return eten_minute(ticker=ticker,pbdate=pbdate,hdrTF=hdrTF,tsTF=tsTF,debugTF=debugTF)

def eten_minute(ticker=None,pbdate=20181120,hdrTF=True,tsTF=True,debugTF=False):
	"""
	Get daily/minute data via eten api
	"""
	if ticker is None:
		return ''
	urx="http://mx68t.etencorp.com:8080/EtenDS/process.php?version=1&objtype=5&extcode={}"
	url=urx.format(ticker)
	if debugTF is True:
		pqint( url, file=sys.stderr)
	try:
		jTmp = pd.read_json(url)['objectData'][0]
	except Exception as e:
		pqint( str(e), file=sys.stderr)
		return {}
	
	#- ARRANGE input data	
	# build output data in datafame
	dx=pd.DataFrame(jTmp['data'])
	dx.loc[:,'ticker']=jTmp['extcode']
	# build date/time as pbdate/epochs column into datafame
	pbdatetime = [datetime.strptime(str(pbdate)+x,'%Y%m%d%H:%M') for x in dx['time'].values]
	dx['epochs'] = [int(x.strftime('%s000')) for x in pbdatetime]
	dx[['open', 'high', 'low', 'close', 'vol']]=dx[['open', 'high', 'low', 'close', 'vol']].astype('float')
	# add datetime index to datafame
	if tsTF is True:
		dx.set_index(pd.DatetimeIndex(pbdatetime),inplace=True)
		dx.index.rename('date',inplace=True)
	# remove NA rows related to [close] data
	dx.rename(columns={'vol':'volume'},inplace=True)
	dx.dropna(subset=['close'],inplace=True)
	# change to prc_temp columns setup
	return dx[['open','high','low','close','volume','epochs','ticker']]

def str2epoch(s,endOfDay=False):
	import re
	if isinstance(s,float):
		s = str(int(s))
	elif isinstance(s,(int,np.integer)):
		s = str(s)
	e = 0
	if len(s)>=10 and s.isdigit():
		e = s[:10]
	elif len(s) in [8,10]:
		s = re.sub('[-/]','',s)
		if len(s)==8 and s.isdigit():
			d = datetime.strptime(s,"%Y%m%d")
			if endOfDay is True:
				d  = d + timedelta(days=1) -  timedelta(seconds=1)
			e = d.strftime("%s")
	return int(e)
	
def yh_rawdata(ticker=None,gap='1m',ranged='1d',debugTF=False):
	"""
	Get daily/minute data via yahoo version 8 api
	"""
	if ticker is None:
		return ''
	if ',' in ranged:
		start, end = ranged.split(',')
		if all([start,end]):
			period1 = str2epoch(start)
			period2 = str2epoch(end,endOfDay=True)
			range_periods = "period1={}&period2={}".format(period1,period2)
		elif  start:
			period1 = str2epoch(start)
			if gap=='1d':
				end = datetime.now().strftime("%Y%m%d")
				period2 = str2epoch(end,endOfDay=True)
			else:
				period2 = (datetime.fromtimestamp(period1)+timedelta(days=1)).strftime("%s")
			range_periods = "period1={}&period2={}".format(period1,period2)
		elif  end:
			period2 = str2epoch(end,endOfDay=True)
			period1 = (datetime.fromtimestamp(period2)+timedelta(days=-1)).strftime("%s")
			#calc beginning time of period1 date
			#if gap=='1d':
			#	d = datetime.fromtimestamp(float(period1))
			#	period1 = datetime.strptime(d.strftime('%Y%m%d'),'%Y%m%d').strftime("%s")
			range_periods = "period1={}&period2={}".format(period1,period2)
		else:
			range_periods = "range=1d"
		urx="https://query1.finance.yahoo.com/v8/finance/chart/{}?region=US&lang=en-US&includePrePost=false&interval={}&{}"
		url=urx.format(ticker,gap,range_periods)
	else:
		urx="https://query1.finance.yahoo.com/v8/finance/chart/{}?region=US&lang=en-US&includePrePost=false&interval={}&range={}"
		url=urx.format(ticker,gap,ranged)
	if debugTF is True:
		pqint( url, file=sys.stderr)
	jX={}
	headers={'Content-Type': 'text/html', 'Accept': 'application/json', 'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36'}
	try:
		#jX = pd.read_json(url)
		ret=requests.Session().get(url,headers=headers)
		#ret=requests.get(url,timeout=5)
		jX=ret.json()
	except Exception as e:
		pqint( str(e), file=sys.stderr)
		return {}
	if 'chart' in jX:
		try:
			jTmp = jX['chart']['result'][0]
		except Exception as e:
			return {}
	return jTmp
	
def yh_data(ticker=None,gap='1m',ranged='1d',hdrTF=True,tsTF=True,pchgTF=False,debugTF=False):
	#- PULL raw data	
	jTmp = yh_rawdata(ticker=ticker,gap=gap,ranged=ranged,debugTF=debugTF)

	#- ARRANGE input data	
	# get timestamp
	if 'indicators' in jTmp and 'adjclose' in jTmp['indicators']:
		adjclose = np.array(jTmp['indicators']['adjclose'][0]['adjclose'])
	if debugTF:
		sys.stderr.write("===== Pulling symbol={}&interval={}&range={}...\n".format(ticker,gap,ranged))
	if 'timestamp' not in jTmp:
		sys.stderr.write("**ERROR pulling symbol={}&interval={}&range={}...\n".format(ticker,gap,ranged))
		pqint( jTmp, file=sys.stderr)
		return {}, {}
	pbtimestamp = np.array(jTmp['timestamp'])
	pbdatetime = [epoch_parser(x,1) for x in pbtimestamp]
	# build output data in datafame
	dx=pd.DataFrame(jTmp['indicators']['quote'][0])
	dx.loc[:,'ticker']=ticker
	# build date/time as pbdate/epochs column into datafame
	if gap[-1]!='m': # for daily data
		if 'adjclose' in locals():
			dx['adjusted'] = adjclose
		dx['pbdate'] = [int(x.strftime("%Y%m%d")) for x in pbdatetime]
	else: # for minute data
		dx['epochs'] = pbtimestamp * 1000
	# add datetime index to datafame
	if tsTF is True:
		dx.set_index(pd.DatetimeIndex(pbdatetime),inplace=True)
		dx.index.rename('date',inplace=True)
	# remove NA rows related to [close] data
	dx.dropna(subset=['close'],inplace=True)
	if pchgTF:
		dx=pchg_calc(dx,jTmp,pchgTF)
	return dx, jTmp

def pchg_calc(dx={},jTmp={},pchgTF=False):
	'''
	add change percent
	'''
	if len(dx)<1:
		return {}
	xClose=xChartClose=None
	if 'meta' in jTmp:
		jm = jTmp['meta']
		if 'chartPreviousClose' in jm and jm['dataGranularity'][-1]!='m':
			xChartClose = jm['chartPreviousClose']
		elif 'previousClose' in jm and jm['dataGranularity'][-1]=='m':
			xClose = jm['previousClose']
	if xClose is not None:
		dx['change'] = dx['close']-xClose
		dx['pchg'] = dx['close']/xClose-1
		dx['xclose'] = xClose
	elif xChartClose is not None and pchgTF==2:
		dx['change'] = dx['close']-xChartClose
		dx['pchg'] = dx['close']/xChartClose-1
		dx['xclose'] = xChartClose
	elif xChartClose is not None:
		dx['xclose'] = dx['close'].shift()
		dx['change'] = dx['close'].diff()
		dx['pchg'] = dx['close'].pct_change()
		dx.loc[dx.index[0],'pchg'] = dx.loc[dx.index[0],'close']/xChartClose - 1
		dx.loc[dx.index[0],'change'] = dx.loc[dx.index[0],'close']-xChartClose
		dx.loc[dx.index[0],'xclose'] = xChartClose
	return dx

def yh_hchg(ticker=None,gap='1m',ranged='1d',hdrTF=True,tsTF=True,debugTF=False,**optx):
	optx.update(pchgTF=True)
	dx, jTmp = yh_data(ticker=ticker,gap=gap,ranged=ranged,hdrTF=hdrTF,tsTF=tsTF,debugTF=debugTF,**optx)
	return dx

def yh_hist(ticker=None,gap='1m',ranged='1d',hdrTF=True,tsTF=True,debugTF=False,**optx):
	pchgTF = getKeyVal(optx,'pchgTF',False)
	dx, jTmp = yh_data(ticker=ticker,gap=gap,ranged=ranged,hdrTF=hdrTF,tsTF=tsTF,debugTF=debugTF,**optx)
	# change to prc_temp columns setup
	clx = dx.columns
	if gap[-1]!='m': # for daily data
		if 'ticker' in clx : # for daily data
			dx.rename(columns={'ticker':'name'},inplace=True)
		else:
			dx['name']=ticker
		clx=dx.columns
		colv=['open','high','low','close','volume','adjusted','pbdate','name']
	else:
		colv=['open','high','low','close','volume','epochs','ticker']
	if pchgTF:
		colv += ['xclose','change','pchg']
	colx = [x for x in colv if x in clx ]
	sys.stderr.write("{}\n{}\n{}\n".format(colx,colv,clx))
	sys.stderr.write("{}\n".format(dx[colx].tail()))
	return dx[colx]

def batch_yh_hist(tkLst=[],opts=None,**optx):
	#- Set input parameters
	if opts is None or len(opts)<1:
		opts, _ = opt_yh_hist([])
	if optx is not None:
		opts.update(optx)
	kys=['gap','ranged','tsTF','pchgTF','debugTF']

	debugTF = getKeyVal(opts,'debugTF',False)
	indexTF = getKeyVal(opts,'indexTF',True)
	output = getKeyVal(opts,'output',None)
	sep = getKeyVal(opts,'sep','|')
	#for ky,va in opts.items():
	#	exec('{}=va'.format(ky))

	hdrTF = True
	if 'funcArg' in opts and opts['funcArg'] in globals():
		funcArg = globals()[opts['funcArg']]
	else:
		funcArg = yh_hist
	if len(tkLst)>0 and tkLst[0]=='-':
		tkLst = sys.stdin.read().split("\n")
	dm=pd.DataFrame()
	for j,ticker in enumerate(tkLst):
		hdrTF = True if j<1 else False
		try:
			df = funcArg(ticker,hdrTF=hdrTF,**subDict(opts,kys))
			if len(df)<1:
				continue
			if output is None or len(output)<1:
				dm= pd.concat([dm,df])
		except Exception as e:
			pqint( "**ERROR: {}.{}\n{}".format(j,ticker,str(e)), file=sys.stderr)
			continue
		if output is not None and 'ticker' not in df:
			df['ticker']=ticker
		if output == 'csv':
			sep = sep.encode().decode('unicode_escape') if sys.version_info.major==3 else sep.decode("string_escape")
			sys.stdout.write(df.to_csv(sep=sep,index=indexTF,header=hdrTF) )
		elif output == 'html':
			sys.stdout.write(df.to_html(index=indexTF) )
		elif output == 'json':
			sys.stdout.write(df.to_json(orient='records') )
		hdrTF = False
	return dm

def opt_yh_hist(argv,retParser=False):
	""" command-line options initial setup
	    Arguments:
		argv:   list arguments, usually passed from sys.argv
		retParser:      OptionParser class return flag, default to False
	    Return: (options, args) tuple if retParser is False else OptionParser class
	"""
	parser = OptionParser(usage="usage: %prog [option] SYMBOL1 ...", version="%prog 0.01",
		description="Pull Price History from YAHOO")
	parser.add_option("","--range",action="store",dest="ranged",default='1d',
		help="range period from now (default: 1d)")
	parser.add_option("","--gap",action="store",dest="gap",default='1m',
		help="interval GAP of data frequency (default: 1m)")
	parser.add_option("-d","--database",action="store",dest="dbname",default="ara",
		help="database (default: ara)")
	parser.add_option("","--host",action="store",dest="hostname",default="localhost",
		help="db host (default: localhost)")
	parser.add_option("-t","--table",action="store",dest="tablename",default="prc_temp_yh",
		help="db tablename (default: prc_temp_yh)")
	parser.add_option("-w","--wmode",action="store",dest="wmode",default="replace",
		help="db table write-mode [replace|append] (default: replace)")
	parser.add_option("-o","--output",action="store",dest="output",
		help="OUTPUT type [csv|html|json] (default: no output)")
	parser.add_option("","--no_datetimeindex",action="store_false",dest="tsTF",default=True,
		help="no datetime index (default: use datetime)")
	parser.add_option("","--show_index",action="store_true",dest="indexTF",default=False,
		help="show index (default: False) Note, OUTPUT ONLY")
	parser.add_option("-s","--sep",action="store",dest="sep",default="|",
		help="output field separator (default: |) Note, OUTPUT ONLY")
	parser.add_option("","--no_database_save",action="store_false",dest="saveDB",default=True,
		help="no save to database (default: save to database)")
	parser.add_option("","--extra_js",action="store",dest="extraJS",
		help="extra JSON in DICT format.")
	parser.add_option("","--extra_qs",action="store",dest="extraQS",
		help="extra GET string format like k1=v1&k2=v2; ")
	parser.add_option("","--extra_xs",action="store",dest="extraXS",
		help="extra excutable string like k1=v1;k2=v2; ")
	parser.add_option("","--debug",action="store_true",dest="debugTF",default=False,
		help="debugging (default: False)")
	(options, args) = parser.parse_args(argv[1:])
	try:
		from _alan_str import extra_opts
		opts = vars(options)
		extra_opts(opts,xkey='extraJS',method='JS',updTF=True)
		extra_opts(opts,xkey='extraQS',method='QS',updTF=True)
		extra_opts(opts,xkey='extraXS',method='XS',updTF=True)
		opts.update(args=args,narg=len(args))
	except Exception as e:
		sys.stderr.write(str(e)+"\n")
	if retParser is True:
		return parser
	return (opts, args)

if __name__ == '__main__':
	(opts, tkLst)=opt_yh_hist(sys.argv)
	batch_yh_hist(tkLst,opts)



#!/usr/bin/env python3
''' Calc intraday 3-major indices movement and most up/down tickers 
of either SP500 with marketcap>50B or DOW30 
and then create a market comment base on jinjia2: 'intraday_headline_cn.j2'
Usage of:
python3 -c "from headline_calc import headline_calc;dd=headline_calc(deltaTolerance=86400);print(dd)"
DB tables required:
PGDB:: sp500_component, dow_component, yh_quote_curr, 
MDG:: yh_quote_curr, market_indicator_quote
Last Mod.,
Fri Jul 30 21:05:01 EDT 2021
'''
import sys
import pandas as pd
from _alan_calc import sqlQuery,subDict

def  get_eqtLst(minMarketCap=50000000000):
	'''
	return equity list for import highlite based on dow_component,sp500_component, yh_quote_curr
	and minimum size (50B) of marketCalp
	'''
	xqTmp = '''SELECT a.ticker FROM sp500_component a, yh_quote_curr b 
		WHERE a.ticker=b.ticker AND 
		(b."marketCap">{} OR a.ticker in (SELECT ticker FROM dow_component))'''
	xqr = xqTmp.format(minMarketCap)
	try:
		eqtLst =  list(sqlQuery(xqr)['ticker'])
	except Exception as e:
		eqtLst=['AAPL','ABT','ACN','ADBE','AMGN','AMZN','AVGO','AXP','BA','BAC','BRK-B','C','CAT','CMCSA','COST','CRM','CSCO','CVX','DHR','DIS','DOW','FB','GOOG','GOOGL','GS','HD','HON','IBM','INTC','JNJ','JPM','KO','LIN','LLY','LMT','MA','MCD','MDT','MMM','MRK','MSFT','NEE','NFLX','NKE','NVDA','ORCL','PEP','PFE','PG','PM','PYPL','SBUX','T','TMO','TRV','TXN','UNH','UNP','UPS','UTX','V','VZ','WBA','WFC','WMT','XOM']
	return eqtLst

def headline_calc(tkLead='',t2Lead='^DJI',idxLst=None,eqtLst=None,np=3,xCol='changePercent',colLst=[],thd=0.05,deltaTolerance=300):
	'''
	return object of {'topLst', 'indexOrder', 'topIndex', 'indexLst', 'topUpDn'}
	Where
	  topIndex: ticker name of lead index defined as 'tkLead'='^GSPC'
	  topUpDn: sign in string UP/FLAT/DOWN within the range of 'thd'=[0.05,-0.05]
	  allUpDn: 1,0,-1 indecis all up/TBD/down
	  topLst: selected 'eqtLst' stock quote info ranked via 'changePercent'
	      w.r.t. the 'sign'/'topUpDn' of 'topIndex'
	  bttmLst: selected 'eqtLst' stock quote info oppsite to topLst
	  indexLst: 'idxLst' stock  quote info listed in the order of 'indexOrder'

	  Note aht topIndex quote info should be in the 'indexLst'
	'''
	from _alan_str import udfStr,find_mdb
	if eqtLst is None or len(eqtLst)<1:
		eqtLst = get_eqtLst()
	if idxLst is None or len(idxLst)<1:
		idxD = {"^GSPC":"標普500","^DJI":"道瓊","^IXIC":"納斯達克"}
		idxLst = list(idxD.keys())
	if colLst is None or len(colLst)<1:
		#colLst=['open','high','low','close','volume','ticker','change','changePercent','pbdate']
		colLst=['close','volume','ticker','change','changePercent','pbdate','pbdt']
	#xqTmp="SELECT * from yh_quote_curr WHERE ticker in ('{}')"

	# get indices quote performance
	#tkStr = "','".join(idxLst)
	#idxRtn = sqlQuery(xqTmp.format(tkStr))[colLst]

	# check if the quote is stale (longer than 300 seconds)
	jobj = dict(ticker={'$in':idxLst})
	idxRtn = find_mdb(jobj,dbname='ara',tablename='market_indicator_quote',dfTF=True)[0][colLst]
	for ky,val in idxD.items():
		try:
			ix = idxRtn.loc[idxRtn['ticker']==ky].index[0]
			idxRtn.loc[ix,'company_cn']=val
		except:
			continue
	if tkLead is None or len(tkLead)<1:
		tkIdx=(abs(idxRtn['changePercent'])).sort_values(ascending=False).index
		tkLead=idxRtn.loc[tkIdx[0],"ticker"]
		t2Lead=idxRtn.loc[tkIdx[1],"ticker"]
	else:
		tkLead='^GSPC'
	topDc=idxRtn.loc[idxRtn['ticker']==tkLead].iloc[0]
	pbdtMod=topDc['pbdt']
	deltaPassed=int(pd.Timedelta(pd.datetime.now()-pbdtMod).total_seconds())
	if deltaPassed>deltaTolerance:
		from yh_chart import yh_spark_hist as ysh;
		ysh(types='quote', debugTF=True)

	# get selected equities quote performance
	#tkStr = "','".join(eqtLst)
	#eqtRtn = sqlQuery(xqTmp.format(tkStr))[colLst]
	jobj = dict(ticker={'$in':eqtLst})
	eqtRtn = find_mdb(jobj,dbname='ara',tablename='yh_quote_curr',dfTF=True)[0][colLst]

	# calc 'topLst' w.r.t. the 'sign'/'topUpDn' of 'topIndex'
	pbdate=topDc['pbdate']
	chgPct=topDc[xCol]
	topUpDn = udfStr(chgPct,['UP','DOWN','FLAT'],thd)
	topSign = udfStr(chgPct,[1,0,-1],thd)
	sign = False if chgPct>=0 else True
	xd = eqtRtn.sort_values(by=[xCol],ascending=sign)
	leadLst = xd.iloc[:np]
	if(xd['changePercent'].iloc[0]*xd['changePercent'].iloc[-1])<0:
		bttmLst = xd.iloc[-1:]
	else:
		bttmLst = []

	# update my lead index in the top level
	dd = dict(topIndex=tkLead)
	dd.update(t2Index=t2Lead)
	dd.update(topUpDn=topUpDn)

	# add  all indices info to idxLst
	dd.update(indexLst=idxRtn.to_dict(orient='records'))
	indexOrder=[x['ticker'] for x in dd['indexLst']]
	dd.update(indexOrder=indexOrder)

	# determine if indices are all Up/Undetermined/Down
	if all([x['changePercent']<0 for x in dd['indexLst']]):
		allUpDn = -1
	elif all([x['changePercent']>0 for x in dd['indexLst']]):
		allUpDn = 1
	else:
		allUpDn = 0
	dd.update(allUpDn=allUpDn)

	# add  topLst 
	if len(leadLst)>0:
		dd.update(topLst=leadLst[colLst].to_dict(orient='records'))
	else:
		dd.update(topLst=[])
	if len(bttmLst)>0:
		dd.update(bttmLst=subDict(bttmLst,colLst).to_dict(orient='records'))
	else:
		dd.update(bttmLst=[])

	# get hiloRecord (based on past 1-year daily change since end date)
	hiloRecord=find_hiloRecord(ticker=tkLead,end=pbdate,days=366)
	dd.update(hiloRecord=hiloRecord)
	dd.update(start=pbdate)
	dd.update(mp3YN=False)

	return dd

def find_hiloRecord(ticker='^GSPC',end=None,days=366,debugTF=False):
	#from record_hilo import find_record_hilo as frh
	from record_hilo import recordHiLo as frh
	from _alan_calc import pull_stock_data as psd
	df=psd(ticker,end=end,days=days,pchgTF=True)
	endDT=df.index[-1]
	jobj=frh(df,endDT,ticker,debugTF=debugTF)
	hiloRecord = jobj['YTD'] if jobj['YTD'] else {}
	return(hiloRecord)

import numpy as np
import datetime
if __name__ == '__main__':
	from _alan_str import jj_fmt
	cdt=datetime.datetime.now()
	hm=int(cdt.strftime("%H00"))
	end_hm=np.clip(hm,900,1600)
	dtr= 86400 if (hm<900 or hm>1600) else 300
	dd= headline_calc(deltaTolerance=dtr)
	sys.stderr.write("{},{}\n".format(end_hm,dd))
	print(jj_fmt("{% include 'intraday_headline_cn.j2' %}",dd,dirname='templates/',outdir="US/mp3_hourly/",end_hm=end_hm ) )
	#print(jj_fmt("{% include 'intraday_briefing_cn.j2' %}",dirname='templates/',outdir="US/mp3_hourly/",end_hm=end_hm,**dd) )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
from optparse import OptionParser

def subDict(myDict,kyLst,reverseTF=False):
	"""
	return a sub-dict of myDict based on the matching [kyLst] keys
	OR
	sub-dict based on the non-matching [kyLst] keys
	"""
	if reverseTF is True: # invert-match, select non-matching [kyLst] keys
		return { ky:myDict[ky] for ky in myDict.keys() if ky not in kyLst }
	else:
		return { ky:myDict[ky] for ky in myDict.keys() if ky in kyLst }

def qs_split(xstr,d1='&',d2='='):
	""" split query string into a dict object """
	d = {k:v for k,v in [lx.split(d2) for lx in xstr.split(d1)]}
	return d

def qs_exec(xstr):
	""" convert commend string into a dict object """
	d = {}
	exec(xstr,globals(),d)
	return d

def extra_opts(opts={},xkey='',method='JS',updTF=True):
	""" 
	update values of extraKey: xkey to DICT: opts 
	based on method: [JS|QS|XS]
	where
		JS: DICT/JSON type object
		QS: k1=v1;k2=v2;... query-like string
	"""	
	import ast	
	if xkey is None or len(xkey)<2:
		xkey = "extra{}".format(method)
	if xkey not in opts or opts[xkey] is None: 
		d = {}
	elif method == 'JS': # additional key/val like POST arguments
		d = ast.literal_eval(opts[xkey])
	elif method == 'QS': # additional key/val like GET arguments
		d = qs_split(opts[xkey])
	else: # additional key/val like python executable string
		d = qs_exec(opts[xkey])
	if updTF is True:
		opts.update(d)
		opts[xkey] = None
	return d

def parse_opt(argv=[],retParser=False,**kwargs):
	""" command-line options initial setup
	    Arguments:
		argv:	list arguments, usually passed from sys.argv
		retParser:	OptionParser class return flag, default to False
	    Return: (options, args) tuple if retParser is False else OptionParser class 
	"""
	optx = dict(usage="usage: %prog [option] SYMBOL1 ...", version="%prog 0.7",
		description="ALAN utilities")
	optx.update({ky:kwargs[ky] for ky in kwargs.keys() if ky in optx.keys()})
	parser = OptionParser(**optx)
	parser.add_option("-f","--file",action="store",dest="filename",
		help="input FILENAME")
	parser.add_option("-s","--sep",action="store",dest="sep",default="|",
		help="field separator (default: |)")
	parser.add_option("-d","--database",action="store",dest="dbname",default="ara",
		help="database DBNAME (default: ara)")
	parser.add_option("","--host",action="store",dest="hostname",default="localhost",
		help="database HOST (default: localhost)")
	parser.add_option("-t","--table",action="store",dest="tablename",
		help="database TABLENAME")
	parser.add_option("","--no_database_save",action="store_false",dest="saveDB",default=True,
		help="no save to database (default: save)")
	parser.add_option("","--start",action="store",dest="start",
		help="START date in YYYYMMDD")
	parser.add_option("","--end",action="store",dest="end",
		help="END date in YYYYMMDD")
	days = int(kwargs['days']) if 'days' in kwargs else 730
	parser.add_option("","--days",action="store",dest="days",default=days,type=int,
		help="number of days from END date (default: {})".format(days))
	parser.add_option("-o","--output",action="store",dest="output",
		help="output type [json|csv|html] (default: None)")
	parser.add_option("-l","--lang",action="store",dest="lang",default="cn",
		help="language (default: cn)")
	parser.add_option("","--extra_js",action="store",dest="extraJS",
		help="extra JSON in DICT format.")
	parser.add_option("","--extra_qs",action="store",dest="extraQS",
		help="extra GET string format like k1=v1&k2=v2; ")
	parser.add_option("","--extra_xs",action="store",dest="extraXS",
		help="extra excutable string like k1=v1;k2=v2; ")
	parser.add_option("","--debug",action="store_true",dest="debugTF",default=False,
		help="debugging (default: False)")
	(options, args) = parser.parse_args(argv[1:])
	try:
		opts = vars(options)
		extra_opts(opts,xkey='extraJS',method='JS',updTF=True)
		extra_opts(opts,xkey='extraQS',method='QS',updTF=True)
		extra_opts(opts,xkey='extraXS',method='XS',updTF=True)
		opts.update(args=args,narg=len(args))
	except Exception as e:
		sys.stderr.write(str(e)+"\n")
	if retParser is True:
		return (opts, parser)
	return (opts, args)

if __name__ == '__main__':
	(opts, args)=parse_opt(sys.argv)
	if (opts['debugTF']==True):
		sys.stderr.write("{} {}\n".format(opts, args))



#!/usr/bin/env python3 
""" program: _alan_pppfcs.py
    Description: ALAN past price performance summary curve fitting
    Example:
	python _alan_pppscf.py AAPL 
	python _alan_pppscf.py DGS10 --src=fred --category=macro
	python _alan_pppscf.py "USSTHPI|Q" --src=fred --category=macro
	printf "SELECT series,freq FROM mapping_series_label WHERE series<>label" | /apps/fafa/bin/psql.sh -d ara -At | python _alan_pppscf.py --src=fred --category=macro --table=macro_pppscf
    Version: 0.66
	def locate_mnmx(vy, locx=0, sign=1, scroll=5):
	def vertex_locator(dy, vx=None, span=5, npar=15, scroll=5, debugTF=False):
	def set_time_index(dg, dh, dtCol=None, parser=epoch_parser):
	def fq_switch(fq_str, fq_dict={'D':21,'W':12,'M':6,'Q':4,'Y':5}):
	def calc_pppchgs(dfr,zs=0.,stdev=1.,avg=0.,fqx='D'):
	def run_pppscf(ticker,opts,pgDB=None):
	def get_db_pg(dbname=None,hostname=None,port=5432):
	def batch_pppscf(tkLst,opts=None,optx=None):
	def opt_pppscf(argv,retParser=False):
    Note:
	find_polyroots() is replaced with vertex_locator()
    Last Mod., Tue Sep  4 18:09:01 EDT 2018
"""
import sys
from optparse import OptionParser
import pandas_datareader.data as web
from sqlalchemy import create_engine
import datetime
import pandas as pd
import numpy as np
import math
from _alan_calc import get_csvdata,pullStockHistory,adjust_fq_days,pqint,ewma_smooth,subDict
from _alan_date import ymd_parser, epoch_parser

def locate_mnmx(vy, locx=0, sign=1, scroll=5):
	""" re-select tx locators to refine the min/max points
	"""
	nobs = len(vy)
	mnmx = max if sign >= 1 else (min if sign <= -1 else None)
	if mnmx is None:
		return locx
	start = max(locx - scroll, 0)
	end = min(locx + scroll, nobs)
	locy = mnmx(vy.iloc[start:end])
	vx = vy.iloc[start:end][vy == locy].index
	tx = vx[-1] if len(vx) else locx
	return tx


def vertex_locator(dy, vx=None, span=5, npar=15, scroll=5, debugTF=False):
	""" locate vertex of array [dy] via polyfit(npar) 
	    	optimize with ewma_smooth(span) and local MinMax locate_mnmx(scroll)  
		return dataframe dh[iptday,actual,fitted,sign]
		where
			span: for running w.moving average [span] period
			npar: for [npar]_th polynomial fitting over time
			scroll: for refine the min-max vertex point +- [scroll] period
			tx are the locators
			ty are the correspoding fitted values
			tz are the correspoding actual values
	"""
	nobs = len(dy)
	if isinstance(dy,pd.Series):
		vy = pd.Series(dy.values)
		vidx = dy.index
	else:
		vy = pd.Series(dy)
		vidx = vy.index
	if vx is None:
		vx = np.arange(nobs)
	vma = ewma_smooth(vy, span) if span > 1 else vy
	params = np.polyfit(vx, vma, npar)
	yfunc = np.poly1d(params)
	ydfunc = yfunc.deriv(m=1)
	vfit = np.polyval(params, vx)
	tx = [ int(x.real) for x in ydfunc.roots[~np.iscomplex(ydfunc.roots)] ]
	tx = list(filter(lambda x: x >= 0 and x <= nobs - 1, tx))
	tx = sorted(set(tx + [0, nobs - 1]))
	ty = list(map(lambda x: yfunc(x), tx))
	ty[0],ty[-1] = (vy.iloc[0], vy.iloc[-1])
	tz = list(vy[tx])
	tsg=[0]
	tsg+= [1 if x>0 else -1 if x <=0 else 0 for x in np.diff(ty)]
	dh=pd.DataFrame(list(zip(tx,ty,tz,tsg)),columns=['iptday','fitted','actual','sign'])

	# re-select tx locator (to refine min/max point
	if scroll > 0:
		tx[1:-1]=[locate_mnmx(vy,locx=x,sign=y,scroll=scroll) for x,y in zip(tx[1:-1],tsg[1:-1]) ]
		tx = sorted(set(tx))
		ty = [yfunc(x) for x in tx]
		ty[0],ty[-1] = (vy.iloc[0], vy.iloc[-1])
		tz = list(vy[tx])
		tsg[1:] = [1 if x>0 else -1 if x <=0 else 0 for x in np.diff(tz)]
		dh=pd.DataFrame(list(zip(tx,ty,tz,tsg)),columns=['iptday','fitted','actual','sign'])
	else:
			pqint( 'MNMX scroll not apply!', file=sys.stderr)
			dh[1:,'sign'] = [1 if x>0 else -1 if x <=0 else 0 for x in np.diff(tz)]
	dh['difday'] = dh['iptday'].diff().fillna(0).astype('int')
	idxname = vidx.name if vidx.name else 'date'
	dh[idxname] = vidx[dh['iptday'].values].values
	dh.index =  vidx[dh['iptday'].values]
	dh.index.name = idxname
	if debugTF is True:
		pqint( "dh in vertex_locator:\n{}".format(dh), file=sys.stderr)

	# no more interpolation, directly apply relative vertices w.r.t. index 
	dg = dh[['actual']]
	dg.columns = ['trend']

	if debugTF is True:
		pqint( "dg in vertex_locator:\n{}".format(dg), file=sys.stderr)
	return (dg, dh)

def set_time_index(dg, dh, dtCol=None, parser=epoch_parser):
	""" rearrange index as time-series
	"""
	dtIdx = list(map(parser, dtCol[dh['iptday']]))
	dh.set_index(pd.DatetimeIndex(dtIdx), inplace=True)
	dg.set_index(pd.DatetimeIndex(map(parser, dtCol)), inplace=True)
	return ( dg, dh)

def fq_switch(fq_str, fq_dict={'D':21,'W':12,'M':6,'Q':4,'Y':5}):
	""" return number of period based on one of specifid frequency strings:[D|W|M|Q|Y]
	"""
	return fq_dict[fq_str] if fq_str in fq_dict else fq_dict["D"]

def calc_pppchgs(dfr,zs=0.,stdev=1.,avg=0.,fqx='D'):
	"""['ticker','label','freq','pdate','xdate','iptday','difday','actual','fitted','slope',
	    'fit_dif','fit_chg','fit_chg_adj','act_dif','act_chg','zcore','fit_chg_zscore','act_chg_zscore',
	    'nthd','sig_daycount','sig_combo']
	"""
	# re-arrange dates 
	dfr['pdate']=[x.strftime("%Y-%m-%d") for x in dfr.date ]
	dfr['xdate']=dfr['pdate'].shift()
	dfr['pbdate']=[int(x.strftime("%Y%m%d")) for x in dfr.date ]

	# assign zs,stdev and avg
	nthd=fq_switch(fqx)
	dfr['avg_std']=zs;dfr['avg']=avg;dfr['stdev']=stdev

 	# calc additional variables
	dfr['fit_dif']=dfr['fitted'].diff();dfr['fit_chg']=dfr['fitted'].pct_change()
	dfr['act_dif']=dfr['actual'].diff();dfr['act_chg']=dfr['actual'].pct_change()
	dfr['slope']=dfr['act_dif']/dfr['difday']
	# Use [fit_chg*mean/std] for zscore to justify last period vaule of changing rate 
	# instead of directly applying: [fit_dif/std]
	dfr['fit_chg_zscore']=dfr['fit_chg']*zs;dfr['act_chg_zscore']=dfr['act_chg']*zs
	mx = dfr['difday'].max();dfr['sig_difday']= list(map(lambda x: 1 if x>=mx else 0,dfr['difday']))
	mx = dfr['fit_chg'].abs().max();dfr['sig_fit_chg']= list(map(lambda x: 1 if abs(x)>=mx else 0,dfr['fit_chg']))
	mx = dfr['act_chg'].abs().max();dfr['sig_act_chg']= list(map(lambda x: 1 if abs(x)>=mx else 0,dfr['act_chg']))
	dfr=dfr.fillna(0) # filling NA with 0 OR backward-filling fillna(method='bfill')
	return dfr

def vertex_mnmx(dh,prc_open,colx='actual'):
	if colx not in dh.columns:
		return {}
	dh.loc[dh.index[0],colx]=prc_open
	dmx = dh.loc[dh[colx] == dh[colx].max()].iloc[0].to_dict()
	dmn = dh.loc[dh[colx] == dh[colx].min()].iloc[0].to_dict()
	dk = []
	if dmx['iptday']>dmn['iptday']:
		dk = dk + [dmn, dmx]
	else:
		dk = dk + [dmx, dmn]
	if dk[0]['iptday'] > dh['iptday'].iloc[0]: 
		dk = [dh.iloc[0].to_dict()] + dk
	if dk[-1]['iptday'] < dh['iptday'].iloc[-1]: 
		dk = dk + [dh.iloc[-1].to_dict()]	
	dk = pd.DataFrame(dk)
	dk['pchg_open'] = [ x/prc_open-1 for x in dk[colx]]
	dk['sign'] = [ 1 if x>0 else -1  for x in dk['pchg_open']]
	dk['difday'] = dk['iptday'].diff()
	return dk

def run_pppscf(ticker,opts=None,pgDB=None,stdinTF=False,debugTF=False):
	"""
	Calc Past Price Performance Summary & CashFlow
	Return dictionary of {'data','dfdr'}
	where
		data: dataframe data of historical prices applied
		dfdr: dataframe the PPP Summary of data
	"""
	if opts is None:
		opts, _ = opt_pppscf([])
	#python3 compatible
	#for ky,va in opts.items():
	#	exec("{}=va".format(ky))
	npar = opts['npar']
	optx = subDict(opts,['sep','start','end','src','days'])
	if stdinTF is True:
		pqint( "==RUNNING get_csvdata({sep},{start},{end},{src},{days})".format(**optx), file=sys.stderr)
		data=get_csvdata('-',pgDB=pgDB,**optx)
	else:
		data=pullStockHistory(ticker,pgDB=pgDB,**optx)
	if isinstance(data.index, (datetime.date, datetime.datetime)) is False:
		if 'pbdate' in data:
			idxpt=[ymd_parser(x,fmt="%Y%m%d") for x in data['pbdate']]
			data.set_index(pd.DatetimeIndex(idxpt),inplace=True)
		elif 'epochs' in data:
			idxpt=[epoch_parser(x) for x in data['epochs']]
			data.set_index(pd.DatetimeIndex(idxpt),inplace=True)
	prc=data['close']
	prc=prc.dropna(axis=0, how='all')

	# polyfit the data at [npar] polynomial
	_, dfdr = vertex_locator(prc,npar=npar,debugTF=debugTF)

	pqint( dfdr, file=sys.stderr)
	pqint( data.head(5), file=sys.stderr)
	# calc additional stats 
	vx=prc[-62:] # vx=prc # using last 3-month as z-score benchmark rather than entire period
	stdev=vx.std();avg=vx.mean();zs=avg/stdev
	dfdr=calc_pppchgs(dfdr,zs=zs,stdev=stdev,avg=avg)
	dfdr=dfdr.drop(['date'],1)
	dfdr['ticker']=ticker
	return {"data":data,"dfdr":dfdr}

def get_db_pg(dbname=None,hostname=None,port=5432):
	engine=None
	if not (dbname is None or hostname is None):
		try:
			dbURL='postgresql://sfdbo@{}:{}/{}'.format(hostname,port,dbname)
			engine = create_engine(dbURL) if(dbURL is not None) else None
		except:
			pqint( "***DB ERROR:", sys.exc_info()[1], file=sys.stderr)
	return engine

def batch_pppscf(tkLst,sector='Technology',va=None,j2ts=None,mp3YN=False,category='stock',wmode='replace',end=None,debugTF=False,hostname='localhost',tablename='ohlc_pppscf',saveDB=False,start=None,j2name=None,freq='D',npar=15,dbname='ara',lang='cn',src='yahoo',plotFlg=False,days=730,output=None,ky='pngname',pngname=None,sep='|',fpTF=False,**optx):
	from lsi_daily import run_comment_pppscf #- test ONLY
	opts=locals()
	opts.pop('tkLst',None)
	opts.pop('optx',None)
	if optx is not None:
		opts.update(optx)
	if opts['category'] != 'stock':
		opts['src'] = 'fred'
	#for ky,va in opts.items():
	#	exec("{}=va".format(ky))
	if debugTF is True:
		pqint("===>", opts, file=sys.stderr)
	pgDB=get_db_pg(dbname=dbname,hostname=hostname)
	wmode='replace'
	dcmt=''
	stdinTF = False
	for j,ticker in enumerate(tkLst):
		if '-' in ticker:
			ticker,_= ticker.split(',')
			stdinTF = True
		if '|' in ticker:
			alst = ticker.split('|')
			dlst = [alst[0],freq,sector,alst[0]]
			(ticker,freq,sector,label) = alst+dlst[len(alst)-len(dlst):]
		else:
			label = ticker
		if freq != 'D':
			nday=adjust_fq_days(days,fq=freq)
			opts['freq']=freq
			opts['days']=nday
		else:
			nday=days
		if debugTF is True:
			pqint( "days:{} after {} adjust.".format(nday,freq), file=sys.stderr)
			pqint( "===RUNNING {}".format(ticker), file=sys.stderr)
		try:
			ret = run_pppscf(ticker,opts,pgDB=pgDB,stdinTF=stdinTF,debugTF=debugTF)
			df=ret['dfdr']
			if debugTF is True:
				pqint( ret['data'].tail(2) , file=sys.stderr)
			if saveDB is True:
				df.to_sql(tablename, pgDB, schema='public', index=False, if_exists=wmode)
				wmode='append'
			else:
				if debugTF is True:
					pqint( df.to_csv(sep="|"), file=sys.stderr)
				# for lang='cn' case ONLY
				if category == 'stock': 
					xqTmp="SELECT ticker,company{0} as label,'D' as freq,'stock' as category, sector{0} as sector FROM mapping_ticker_cik WHERE ticker='{1}'"
				else:
					xqTmp="SELECT series as ticker,label{0} as label, freq,category, category{0} as sector FROM mapping_series_label WHERE series='{1}'"

				dx=pd.read_sql(xqTmp.format("_cn" if lang=="cn" else "", ticker),pgDB)
				if dx.shape[0]<1:
					dx=pd.DataFrame([dict(ticker=ticker,label=label,freq=freq,category=category,sector=sector)])
				fp=df.merge(dx,on='ticker')
				j2ts=j2ts if j2ts is not None else open(j2name).read() if j2name is not None else None
				dcmt = run_comment_pppscf(ticker=ticker,fp=fp,pgDB=pgDB,lang=lang,ts=j2ts)
				pqint(dcmt, file=sys.stdout)
		except Exception as e:
			pqint( '**ERROR {} [{}]:{}'.format("batch_pppscf()",ticker,str(e)), file=sys.stderr)
			continue

	if pgDB is not None:
		pgDB.dispose()
	global gData
	try:
		from lsi_daily import gData
	except:
		gData={}
	return dcmt if fpTF is False else (dcmt, fp)

def opt_pppscf(argv,retParser=False):
	""" command-line options initial setup
	    Arguments:
		argv:   list arguments, usually passed from sys.argv
		retParser:      OptionParser class return flag, default to False
	    Return: (options, args) tuple if retParser is False else OptionParser class
	"""
	parser = OptionParser(usage="usage: %prog [option] SYMBOL1 ...", version="%prog 0.65",
		description="ALAN Past Price Performance Summary Curve Fitting") 
	parser.add_option("-s","--sep",action="store",dest="sep",default="|",
		help="field separator (default: |)")
	parser.add_option("","--start",action="store",dest="start",
		help="start YYYY-MM-DD (default: 3-years-ago)")
	parser.add_option("","--end",action="store",dest="end",
		help="end YYYY-MM-DD (default: today)")
	parser.add_option("","--days",action="store",dest="days",default=730,type=int,
		help="number of days from endDate (default: 730)")
	parser.add_option("-d","--database",action="store",dest="dbname",default="ara",
		help="database (default: ara)")
	parser.add_option("","--host",action="store",dest="hostname",default="localhost",
		help="db host (default: localhost)")
	parser.add_option("-t","--table",action="store",dest="tablename",default="ohlc_pppscf",
		help="db tablename (default: ohlc_pppscf)")
	parser.add_option("-w","--wmode",action="store",dest="wmode",default="replace",
		help="db table write-mode [replace|append|fail] (default: replace)")
	parser.add_option("-o","--output",action="store",dest="output",
		help="output type (default: None)")
	parser.add_option("","--src",action="store",dest="src",default="yahoo",
		help="source [fred|yahoo](default: yahoo)")
	parser.add_option("","--category",action="store",dest="category",default="stock",
		help="category [stock|macro|interest_rate](default: stock)")
	parser.add_option("","--freq",action="store",dest="freq",default="D",
		help="frequency period [D|W|M|Q|Y](default: D)")
	parser.add_option("","--sector",action="store",dest="sector",default="Technology",
		help="sector (default: Technology)")
	parser.add_option("","--npar",action="store",dest="npar",default=15,type="int",
		help="nth polynomial (default: 15)")
	parser.add_option("","--plot",action="store_true",dest="plotFlg",default=False,
		help="show plot(default: False)")
	parser.add_option("","--no_database_save",action="store_false",dest="saveDB",default=True,
		help="no save to database (default: save to database)")
	parser.add_option("-l","--lang",action="store",dest="lang",default="cn",
		help="db language mode [cn|en] (default: cn)")
	parser.add_option("","--png",action="store",dest="pngname",
		help="graph name (default: None)")
	parser.add_option("","--j2ts",action="store",dest="j2ts",
		help="jinja2 template script, (default: None).")
	parser.add_option("","--j2name",action="store",dest="j2name",
		help="jinja2 template file (default: None). ONLY valid if j2ts is None")
	parser.add_option("","--use_mp3",action="store_true",dest="mp3YN",default=False,
		help="comment use mp3 style")
	parser.add_option("","--debug",action="store_true",dest="debugTF",default=False,
		help="debugging (default: False)")
	(options, args) = parser.parse_args(argv[1:])
	if retParser is True:
		return parser
	return (vars(options), args)

if __name__ == '__main__':
	(opts, args)=opt_pppscf(sys.argv)
	if len(args)==0 or args[0]=='-' :
		tkLst = sys.stdin.read().strip().split("\n")
	else :
		tkLst = args
		opts['saveDB']=False
	batch_pppscf(tkLst, **opts)
